@@ -3,7 +3,7 @@
 
 #include <asm/xenomai/features.h>
 
-#if __LINUX_ARM_ARCH__ >= 4
+#if __LINUX_ARM_ARCH__ >= 4 && (!defined(CONFIG_THUMB2_KERNEL) || !defined(CONFIG_FTRACE))
 static inline __attribute__((__const__)) unsigned long long
 rthal_arm_nodiv_ullimd(const unsigned long long op,
 		       const unsigned long long frac,
@@ -32,7 +32,7 @@ rthal_arm_nodiv_llimd(const long long op,
 
 #include <asm-generic/xenomai/arith.h>
 
-#if __LINUX_ARM_ARCH__ >= 4
+#if __LINUX_ARM_ARCH__ >= 4 && (!defined(CONFIG_THUMB2_KERNEL) || !defined(CONFIG_FTRACE))
 #define rthal_arm_nodiv_ullimd_str			\
 	"umull %[tl], %[rl], %[opl], %[fracl]\n\t"	\
 	"umull %[rm], %[rh], %[oph], %[frach]\n\t"	\
