@@ -13,8 +13,10 @@
 #include <asm-generic/xenomai/bits/bind.h>
 #include <asm/xenomai/syscall.h>
 
-__attribute__((weak)) struct __xn_tscinfo __xn_tscinfo = {
-  type: -1
+struct __xn_full_tscinfo __xn_tscinfo = {
+	.kinfo = {
+		.type = -1,
+	},
 };
 
 static inline void xeno_arm_features_check(struct xnfeatinfo *finfo)
@@ -25,7 +27,7 @@ static inline void xeno_arm_features_check(struct xnfeatinfo *finfo)
 	int err, fd;
 	void *addr;
 
-	if (__xn_tscinfo.type != -1)
+	if (__xn_tscinfo.kinfo.type != -1)
 		return;
 
 	err = XENOMAI_SYSCALL2(__xn_sys_arch,
@@ -41,12 +43,12 @@ static inline void xeno_arm_features_check(struct xnfeatinfo *finfo)
 
 	page_size = sysconf(_SC_PAGESIZE);
 
-	switch(__xn_tscinfo.type) {
+	switch(__xn_tscinfo.kinfo.type) {
 #if CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_KUSER
-	case __XN_TSC_TYPE_FREERUNNING:
-	case __XN_TSC_TYPE_FREERUNNING_COUNTDOWN:
-	case __XN_TSC_TYPE_FREERUNNING_FAST_WRAP:
-	case __XN_TSC_TYPE_DECREMENTER:
+	default:
+		__xn_tscinfo.kuser_tsc_get =
+			(rdtsc_t *)(0xffff1004 -
+				    ((*(unsigned *)(0xffff0ffc) + 3) << 5));
 		goto domap;
 
 #elif CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_FREERUNNING		\
@@ -55,7 +57,7 @@ static inline void xeno_arm_features_check(struct xnfeatinfo *finfo)
 	case __XN_TSC_TYPE_FREERUNNING:
 	case __XN_TSC_TYPE_FREERUNNING_COUNTDOWN:
 #if CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_FREERUNNING_FAST_WRAP
-		if (__xn_tscinfo.mask >= ((1 << 28) - 1)) {
+		if (__xn_tscinfo.kinfo.mask >= ((1 << 28) - 1)) {
 			fprintf(stderr, "Hardware tsc is not a fast wrapping"
 				" one, select the correct platform, or fix\n"
 				"configure.in\n");
@@ -64,26 +66,30 @@ static inline void xeno_arm_features_check(struct xnfeatinfo *finfo)
 #endif /* __XN_TSC_TYPE_FREERUNNING_FAST_WRAP */
 		goto domap;
 
+	default:
+		fprintf(stderr,
+			"Xenomai: kernel/user tsc emulation mismatch.\n");
+		exit(EXIT_FAILURE);
+		break;
 #elif CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_DECREMENTER
 	case __XN_TSC_TYPE_DECREMENTER:
 		goto domap;
 
+	default:
+		fprintf(stderr,
+			"Xenomai: kernel/user tsc emulation mismatch.\n");
+		exit(EXIT_FAILURE);
+		break;
 #endif /* CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_DECREMENTER */
 	case __XN_TSC_TYPE_NONE:
 	  error:
 		fprintf(stderr, "Xenomai: Your board/configuration does not"
 			" allow tsc emulation in user-space: %d\n", err);
 		exit(EXIT_FAILURE);
-		break;
-
-	default:
-		fprintf(stderr,
-			"Xenomai: kernel/user tsc emulation mismatch.\n");
-		exit(EXIT_FAILURE);
 	}
 
   domap:
-	phys_addr = (unsigned long) __xn_tscinfo.counter;
+	phys_addr = (unsigned long) __xn_tscinfo.kinfo.counter;
 
 	addr = mmap(NULL, page_size, PROT_READ, MAP_SHARED,
 		    fd, phys_addr & ~(page_size - 1));
@@ -92,7 +98,7 @@ static inline void xeno_arm_features_check(struct xnfeatinfo *finfo)
 		exit(EXIT_FAILURE);
 	}
 
-	__xn_tscinfo.counter =
+	__xn_tscinfo.kinfo.counter =
 		((volatile unsigned *)
 		 ((char *) addr + (phys_addr & (page_size - 1))));
 
