@@ -61,7 +61,8 @@
 #define __LINUX_ARM_ARCH__ 5
 #endif /* armv5 */
 
-#if defined(__ARM_ARCH_6__) || defined(__ARM_ARCH_6K__)
+#if defined(__ARM_ARCH_6__) || defined(__ARM_ARCH_6K__) \
+	|| defined(__ARM_ARCH_6Z__) || defined(__ARM_ARCH_6ZK__)
 #define __LINUX_ARM_ARCH__ 6
 #endif /* armv6 */
 
@@ -73,9 +74,7 @@
 #error "Could not find current ARM architecture"
 #endif
 
-#if __LINUX_ARM_ARCH__ >= 6 || !defined(CONFIG_SMP)
 #define CONFIG_XENO_FASTSYNCH 1
-#endif
 
 #if CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_KUSER
 #define CONFIG_XENO_ARM_KUSER_TSC 1
