@@ -241,23 +241,25 @@ struct __xn_tscinfo {
 };
 
 #ifndef __KERNEL__
-extern struct __xn_tscinfo __xn_tscinfo;
+/* Putting kuser_tsc_get and kinfo.counter in the same struct results
+   in less operations in PIC code, thus optimizes */
+typedef unsigned long long rdtsc_t(volatile unsigned *vaddr);
+struct __xn_full_tscinfo {
+	struct __xn_tscinfo kinfo;
+	rdtsc_t *kuser_tsc_get;
+};
+extern struct __xn_full_tscinfo __xn_tscinfo;
 
 #ifdef CONFIG_XENO_ARM_TSC_TYPE
 static inline unsigned long long __xn_rdtsc(void)
 {
 #if CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_KUSER
-	typedef unsigned long long rdtsc_t(volatile unsigned *vaddr);
-	rdtsc_t *const kuser_tsc_get =
-		(rdtsc_t *)(0xffff1004 -
-			    ((*(unsigned *)(0xffff0ffc) + 3) << 5));
-
-	return kuser_tsc_get(__xn_tscinfo.counter);
+	return __xn_tscinfo.kuser_tsc_get(__xn_tscinfo.kinfo.counter);
 
 #elif CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_FREERUNNING
-	volatile unsigned long long *const tscp = __xn_tscinfo.tsc;
-	volatile unsigned *const counterp = __xn_tscinfo.counter;
-	const unsigned mask = __xn_tscinfo.mask;
+	volatile unsigned long long *const tscp = __xn_tscinfo.kinfo.tsc;
+	volatile unsigned *const counterp = __xn_tscinfo.kinfo.counter;
+	const unsigned mask = __xn_tscinfo.kinfo.mask;
 	register unsigned long long result;
 	unsigned counter;
 
@@ -270,9 +272,9 @@ static inline unsigned long long __xn_rdtsc(void)
 	return (result & ~((unsigned long long) mask)) | (counter & mask);
 
 #elif CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_FREERUNNING_COUNTDOWN
-	volatile unsigned long long *const tscp = __xn_tscinfo.tsc;
-	volatile unsigned *const counterp = __xn_tscinfo.counter;
-	const unsigned mask = __xn_tscinfo.mask;
+	volatile unsigned long long *const tscp = __xn_tscinfo.kinfo.tsc;
+	volatile unsigned *const counterp = __xn_tscinfo.kinfo.counter;
+	const unsigned mask = __xn_tscinfo.kinfo.mask;
 	register unsigned long long result;
 	unsigned counter;
 
@@ -285,9 +287,9 @@ static inline unsigned long long __xn_rdtsc(void)
 	return (result & ~((unsigned long long) mask)) | (counter & mask);
 
 #elif CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_FREERUNNING_FAST_WRAP
-	volatile unsigned long long *const tscp = __xn_tscinfo.tsc;
-	volatile unsigned *const counterp = __xn_tscinfo.counter;
-	const unsigned mask = __xn_tscinfo.mask;
+	volatile unsigned long long *const tscp = __xn_tscinfo.kinfo.tsc;
+	volatile unsigned *const counterp = __xn_tscinfo.kinfo.counter;
+	const unsigned mask = __xn_tscinfo.kinfo.mask;
 	register unsigned long long after, before;
 	unsigned counter;
 
@@ -303,10 +305,10 @@ static inline unsigned long long __xn_rdtsc(void)
 	return (before & ~((unsigned long long) mask)) | (counter & mask);
 
 #elif CONFIG_XENO_ARM_TSC_TYPE == __XN_TSC_TYPE_DECREMENTER
-	volatile unsigned long long *const tscp = __xn_tscinfo.tsc;
-	volatile unsigned *const counterp = __xn_tscinfo.counter;
-	volatile unsigned *const last_cntp = __xn_tscinfo.last_cnt;
-	const unsigned mask = __xn_tscinfo.mask;
+	volatile unsigned long long *const tscp = __xn_tscinfo.kinfo.tsc;
+	volatile unsigned *const counterp = __xn_tscinfo.kinfo.counter;
+	volatile unsigned *const last_cntp = __xn_tscinfo.kinfo.last_cnt;
+	const unsigned mask = __xn_tscinfo.kinfo.mask;
 	register unsigned long long after, before;
 	unsigned counter, last_cnt;
 
