@@ -31,13 +31,8 @@ static inline void xnarch_program_timer_shot(unsigned long delay)
 	rthal_timer_program_shot(delay);
 }
 
-static inline int xnarch_send_timer_ipi(xnarch_cpumask_t mask)
+static inline void xnarch_send_timer_ipi(xnarch_cpumask_t mask)
 {
-#ifdef CONFIG_SMP
-	return -1;		/* FIXME */
-#else /* ! CONFIG_SMP */
-	return 0;
-#endif /* CONFIG_SMP */
 }
 
 #endif /* !_XENO_ASM_BLACKFIN_BITS_TIMER_H */
