@@ -4,9 +4,6 @@
 #include <linux/pci_ids.h>
 
 /* SMI */
-#ifndef PCI_DEVICE_ID_INTEL_82801EB_0
-#define PCI_DEVICE_ID_INTEL_82801EB_0 0x24d0
-#endif
 #ifndef PCI_DEVICE_ID_INTEL_ESB2_0
 #define PCI_DEVICE_ID_INTEL_ESB2_0 0x2670
 #endif
