@@ -25,6 +25,7 @@
 #error "Pure kernel header included from user-space!"
 #endif
 
+#include <linux/ipipe.h>
 #include <linux/version.h>
 #include <linux/module.h>
 #include <linux/slab.h>
@@ -33,6 +34,14 @@
 #endif /* CONFIG_XENO_OPT_HOSTRT || __IPIPE_FEATURE_REQUEST_TICKDEV */
 #include <asm/io.h>
 
+#ifndef CONFIG_IPIPE_CORE
+#define IPIPE_CORE_APIREV  0
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,4,0)
+#include <asm/system.h>
+#endif /* kernel < 3.4.0 */
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 
 #include <linux/wrapper.h>
@@ -90,10 +99,6 @@
 #define __copy_to_user_inatomic		__copy_to_user
 #define __copy_from_user_inatomic	__copy_from_user
 
-/* Seqfiles */
-#define SEQ_START_TOKEN ((void *)1)
-#define SEQ_SKIP 	0	/* not implemented. */
-
 /* Sched and process flags */
 #define MAX_RT_PRIO 100
 #define task_cpu(p) ((p)->processor)
@@ -389,7 +394,7 @@ static inline void *kzalloc(size_t size, int flags)
 #define wrap_remap_io_page_range(vma,from,to,size,prot)  ({		\
     (vma)->vm_page_prot = pgprot_noncached((vma)->vm_page_prot);	\
     /* Sets VM_RESERVED | VM_IO | VM_PFNMAP on the vma. */		\
-    remap_pfn_range(vma,from,(to) >> PAGE_SHIFT,size,prot);		\
+    remap_pfn_range(vma,from,(to) >> PAGE_SHIFT,size,pgprot_noncached(prot));		\
     })
 #define wrap_remap_kmem_page_range(vma,from,to,size,prot)  ({		\
     /* Sets VM_RESERVED | VM_IO | VM_PFNMAP on the vma. */		\
@@ -420,8 +425,20 @@ static inline void *kzalloc(size_t size, int flags)
 #define pgprot_noncached(p) (p)
 #endif /* !pgprot_noncached */
 
-#define wrap_switch_mm(prev,next,task)	\
-    switch_mm(prev,next,task)
+#if IPIPE_CORE_APIREV >= 2
+#define wrap_switch_mm(prev, next, tsk) \
+	ipipe_switch_mm_head(prev, next, tsk)
+#elif IPIPE_CORE_APIREV == 1
+#define wrap_switch_mm(prev, next, tsk) \
+	__switch_mm(prev, next, tsk)
+#elif defined(__IPIPE_FEATURE_HARDENED_SWITCHMM)
+#define wrap_switch_mm(prev, next, tsk) \
+	__switch_mm(prev, next, tsk)
+#else
+#define wrap_switch_mm(prev, next, tsk) \
+	switch_mm(prev, next, tsk)
+#endif
+
 #define wrap_enter_lazy_tlb(mm,task)	\
     enter_lazy_tlb(mm,task)
 
@@ -487,6 +504,17 @@ static inline void *kzalloc(size_t size, int flags)
 #define IRQF_SHARED			SA_SHIRQ
 #endif /* < 2.6.18 */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
+#define clamp(val, min, max) ({			\
+	typeof(val) __val = (val);		\
+	typeof(min) __min = (min);		\
+	typeof(max) __max = (max);		\
+	(void) (&__val == &__min);		\
+	(void) (&__val == &__max);		\
+	__val = __val < __min ? __min: __val;	\
+	__val > __max ? __max: __val; })
+#endif /* < 2.6.25 */
+
 #if defined(CONFIG_LTT) || \
     (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,31) && defined(CONFIG_MARKERS))
 
@@ -637,7 +665,11 @@ static inline void wrap_proc_dir_entry_owner(struct proc_dir_entry *entry)
 
 #ifndef DEFINE_SEMAPHORE
 /* Legacy DECLARE_MUTEX vanished in 2.6.37 */
-#define DEFINE_SEMAPHORE(sem) DECLARE_MUTEX(sem)
+#define DEFINE_BINARY_SEMAPHORE(sem) DECLARE_MUTEX(sem)
+#elif defined(CONFIG_PREEMPT_RT)
+#define DEFINE_BINARY_SEMAPHORE(sem) DEFINE_SEMAPHORE(sem, 1)
+#else
+#define DEFINE_BINARY_SEMAPHORE(sem) DEFINE_SEMAPHORE(sem)
 #endif
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37) && defined(CONFIG_GENERIC_HARDIRQS)
@@ -675,4 +707,52 @@ static inline void wrap_proc_dir_entry_owner(struct proc_dir_entry *entry)
 	})
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,4,0)
+
+#include <linux/sched.h>
+#include <linux/mm.h>
+#include <linux/smp.h>
+
+#ifndef cpu_online_map
+#define cpu_online_mask (&cpu_online_map)
+#endif /* !cpu_online_map */
+#ifndef cpu_online
+#ifdef CONFIG_SMP
+#define cpu_online(cpu)	(cpu_online_map & (1UL << (cpu)))
+#else /* !CONFIG_SMP */
+#define cpu_online(cpu)	((cpu) == 0)
+#endif /* !CONFIG_SMP */
+#endif /* !cpu_online */
+
+static inline
+unsigned long vm_mmap(struct file *file, unsigned long addr,
+	unsigned long len, unsigned long prot,
+	unsigned long flag, unsigned long offset)
+{
+	struct mm_struct *mm = current->mm;
+	unsigned long ret;
+
+	down_write(&mm->mmap_sem);
+	ret = do_mmap(file, addr, len, prot, flag, offset);
+	up_write(&mm->mmap_sem);
+
+	return ret;
+}
+
+#endif /* LINUX_VERSION_CODE < 3.4.0 */
+
+#include <linux/seq_file.h>
+#ifndef SEQ_START_TOKEN
+#define SEQ_START_TOKEN ((void *)1)
+#endif
+#ifndef SEQ_SKIP
+#define SEQ_SKIP 	0	/* not implemented. */
+#endif
+
+#if IPIPE_CORE_APIREV >= 2
+#define wrap_select_timers(mask) ipipe_select_timers(mask)
+#elif IPIPE_CORE_APIREV == 1
+#define wrap_select_timers(mask) ipipe_timers_request()
+#endif
+
 #endif /* _XENO_ASM_GENERIC_WRAPPERS_H */
