@@ -39,11 +39,12 @@ void xnpod_schedule_handler(void);
 
 static rthal_trap_handler_t xnarch_old_trap_handler;
 
-static int xnarch_trap_fault(unsigned event, unsigned domid, void *data)
+static int xnarch_trap_fault(unsigned event, rthal_pipeline_stage_t *stage,
+			     void *data)
 {
 	xnarch_fltinfo_t fltinfo;
 	fltinfo.exception = event;
-	fltinfo.regs = (struct pt_regs *)data;
+	fltinfo.regs = data;
 	return xnpod_trap_fault(&fltinfo);
 }
 
@@ -105,6 +106,8 @@ static inline int xnarch_init(void)
 static inline void xnarch_exit(void)
 {
 	rthal_trap_catch(xnarch_old_trap_handler);
+	rthal_virtualize_irq(&rthal_domain,
+			     xnarch_escalation_virq, NULL, NULL, NULL, 0);
 	rthal_free_virq(xnarch_escalation_virq);
 	rthal_exit();
 }
