@@ -31,7 +31,12 @@
 
 #include <asm-generic/xenomai/hal.h>	/* Read the generic bits. */
 
+#ifdef CONFIG_IPIPE_CORE
+#include <linux/ipipe_tickdev.h>
+#define RTHAL_TIMER_DEVICE	ipipe_timer_name()
+#else
 #define RTHAL_TIMER_DEVICE	"decrementer"
+#endif
 #define RTHAL_CLOCK_DEVICE	"timebase"
 
 typedef unsigned long long rthal_time_t;
@@ -49,7 +54,6 @@ static inline __attribute_const__ unsigned long ffnz(unsigned long ul)
 
 #ifndef __cplusplus
 #include <linux/irq.h>
-#include <asm/system.h>
 #include <asm/time.h>
 #include <asm/timex.h>
 #include <asm/xenomai/atomic.h>
@@ -57,8 +61,17 @@ static inline __attribute_const__ unsigned long ffnz(unsigned long ul)
 
 #define RTHAL_TIMER_IRQ		IPIPE_TIMER_VIRQ
 #ifdef CONFIG_SMP
-#define RTHAL_TIMER_IPI		IPIPE_SERVICE_IPI3
+#define RTHAL_TIMER_IPI		RTHAL_HRTIMER_IPI
+#ifndef CONFIG_IPIPE_CORE
+/*
+ * RTHAL_HOST_TIMER_IPI is only needed with old kernels with no
+ * support for generic clock events. So either we have a legacy kernel
+ * with a legacy pipeline, or we are running over a recent pipeline
+ * core (i.e. >= linux kernel 3.1) therefore we do have generic clock
+ * events, which means we don't need the host timer IPI.
+ */
 #define RTHAL_HOST_TIMER_IPI	IPIPE_SERVICE_IPI4
+#endif
 #endif /* CONFIG_SMP */
 
 #define DECREMENTER_MAX		0x7fffffff
@@ -75,6 +88,9 @@ static inline unsigned long long rthal_rdtsc(void)
 
 static inline void rthal_timer_program_shot(unsigned long delay)
 {
+#ifdef CONFIG_IPIPE_CORE
+	ipipe_timer_set(delay);
+#else /* !CONFIG_IPIPE_CORE */
 	if (delay < 3)
 		rthal_schedule_irq_head(RTHAL_TIMER_IRQ);
 	else {
@@ -90,15 +106,20 @@ static inline void rthal_timer_program_shot(unsigned long delay)
 		set_dec((int)delay);
 #endif /* CONFIG_40x */
 	}
+#endif /* !CONFIG_IPIPE_CORE */
 }
 
 static inline struct mm_struct *rthal_get_active_mm(void)
 {
 #ifdef CONFIG_XENO_HW_UNLOCKED_SWITCH
-	return per_cpu(ipipe_active_mm, smp_processor_id());
+#ifdef CONFIG_IPIPE_CORE
+	return __this_cpu_read(ipipe_percpu.active_mm);
 #else
-	return current->active_mm;
+	return per_cpu(ipipe_active_mm, smp_processor_id());
 #endif
+#else  /* !CONFIG_XENO_HW_UNLOCKED_SWITCH */
+	return current->active_mm;
+#endif  /* !CONFIG_XENO_HW_UNLOCKED_SWITCH */
 }
 
     /* Private interface -- Internal use only */
