@@ -26,14 +26,17 @@
 
 static inline void xnarch_program_timer_shot(unsigned long delay)
 {
+#if IPIPE_CORE_APIREV < 2
 	unsigned long d;
 	d = rthal_imuldiv_ceil(delay, RTHAL_TIMER_FREQ, RTHAL_CLOCK_FREQ);
 	rthal_timer_program_shot(d);
+#else
+	rthal_timer_program_shot(delay);
+#endif
 }
 
-static inline int xnarch_send_timer_ipi(xnarch_cpumask_t mask)
+static inline void xnarch_send_timer_ipi(xnarch_cpumask_t mask)
 {
-	return -1;	/* No SMP support */
 }
 
 #endif /* !_XENO_ASM_SH_BITS_TIMER_H */
