@@ -70,7 +70,7 @@ static inline void xnarch_setup_mayday_page(void *page)
 	 * Also note that if SEP is present, we always assume NPTL on
 	 * the user side.
 	 */
-	static const struct {
+	static const struct __attribute__ ((__packed__)) {
 		struct __attribute__ ((__packed__)) {
 			u8 op;
 			u32 imm;
@@ -94,7 +94,7 @@ static inline void xnarch_setup_mayday_page(void *page)
 		.bug = 0x0b0f,
 	};
 
-	static const struct {
+	static const struct __attribute__ ((__packed__)) {
 		struct __attribute__ ((__packed__)) {
 			u8 op;
 			u32 imm;
@@ -127,6 +127,7 @@ static inline void xnarch_handle_mayday(struct xnarchtcb *tcb,
 					struct pt_regs *regs,
 					unsigned long tramp)
 {
+	tcb->mayday.esp = regs->x86reg_sp;
 	tcb->mayday.eip = regs->x86reg_ip;
 	tcb->mayday.eax = regs->x86reg_ax;
 	regs->x86reg_ip = tramp;
@@ -137,6 +138,7 @@ static inline void xnarch_fixup_mayday(struct xnarchtcb *tcb,
 {
 	regs->x86reg_ip = tcb->mayday.eip;
 	regs->x86reg_ax = tcb->mayday.eax;
+	regs->x86reg_sp = tcb->mayday.esp;
 }
 
 #endif /* XNARCH_HAVE_MAYDAY */
