@@ -26,6 +26,7 @@
 
 static inline void xnarch_program_timer_shot(unsigned long delay)
 {
+#if IPIPE_CORE_APIREV < 2
 	/*
 	 * Even though we may use a 64 bits delay here, we voluntarily
 	 * limit to 32 bits, 4 billions ticks should be enough for
@@ -33,15 +34,16 @@ static inline void xnarch_program_timer_shot(unsigned long delay)
 	 * handler would simply occur after 4 billions ticks.
 	 */
 	rthal_timer_program_shot(rthal_imuldiv_ceil
-				 (delay, RTHAL_TIMER_FREQ, RTHAL_CPU_FREQ));
+				 (delay, RTHAL_TIMER_FREQ, RTHAL_CLOCK_FREQ));
+#else
+	rthal_timer_program_shot(delay);
+#endif
 }
 
-static inline int xnarch_send_timer_ipi(xnarch_cpumask_t mask)
+static inline void xnarch_send_timer_ipi(xnarch_cpumask_t mask)
 {
 #ifdef CONFIG_SMP
-	return rthal_send_ipi(RTHAL_APIC_TIMER_IPI, mask);
-#else /* ! CONFIG_SMP */
-	return 0;
+	rthal_send_ipi(RTHAL_TIMER_IPI, mask);
 #endif /* CONFIG_SMP */
 }
 
