@@ -24,7 +24,11 @@
 #define _XENO_ASM_X86_HAL_64_H
 
 #define RTHAL_ARCH_NAME			"x86_64"
+#ifdef CONFIG_IPIPE_CORE
+#define RTHAL_TIMER_DEVICE		ipipe_timer_name()
+#else /* !I-ipipe core */
 #define RTHAL_TIMER_DEVICE		"lapic"
+#endif /* !I-ipipe core */
 #define RTHAL_CLOCK_DEVICE		"tsc"
 
 #include <asm/xenomai/wrappers.h>
@@ -39,7 +43,6 @@ static inline __attribute_const__ unsigned long ffnz(unsigned long ul)
 }
 
 #ifndef __cplusplus
-#include <asm/system.h>
 #include <asm/io.h>
 #include <asm/timex.h>
 #include <asm/processor.h>
@@ -49,13 +52,20 @@ static inline __attribute_const__ unsigned long ffnz(unsigned long ul)
 #include <asm/xenomai/atomic.h>
 #include <asm/xenomai/smi.h>
 
-#define RTHAL_APIC_TIMER_VECTOR		RTHAL_SERVICE_VECTOR3
-#define RTHAL_APIC_TIMER_IPI		RTHAL_SERVICE_IPI3
-#define RTHAL_APIC_ICOUNT		((RTHAL_TIMER_FREQ + HZ/2)/HZ)
-#define RTHAL_TIMER_IRQ			RTHAL_APIC_TIMER_IPI
-#define RTHAL_NMICLK_FREQ		RTHAL_CPU_FREQ
-#define RTHAL_HOST_TICK_IRQ		ipipe_apic_vector_irq(LOCAL_TIMER_VECTOR)
-#define RTHAL_BCAST_TICK_IRQ		0
+#ifdef CONFIG_IPIPE_CORE
+#define RTHAL_TIMER_IRQ		__ipipe_hrtimer_irq
+#define RTHAL_HOST_TICK_IRQ	__ipipe_hrtimer_irq
+#define RTHAL_TIMER_IPI		RTHAL_HRTIMER_IPI
+#else /* !I-ipipe core */
+#define RTHAL_HRTIMER_VECTOR	IPIPE_SERVICE_VECTOR0
+#define RTHAL_APIC_TIMER_VECTOR	RTHAL_HRTIMER_VECTOR
+#define RTHAL_TIMER_IPI		RTHAL_HRTIMER_IPI
+#define RTHAL_APIC_ICOUNT	((RTHAL_TIMER_FREQ + HZ/2)/HZ)
+#define RTHAL_TIMER_IRQ		RTHAL_TIMER_IPI
+#define RTHAL_NMICLK_FREQ	RTHAL_CPU_FREQ
+#define RTHAL_HOST_TICK_IRQ	ipipe_apic_vector_irq(LOCAL_TIMER_VECTOR)
+#define RTHAL_BCAST_TICK_IRQ	0
+#endif /* !I-ipipe core */
 
 static inline void rthal_grab_control(void)
 {
@@ -77,6 +87,9 @@ static inline unsigned long long rthal_rdtsc(void)
 
 static inline void rthal_timer_program_shot(unsigned long delay)
 {
+#ifdef CONFIG_IPIPE_CORE
+	ipipe_timer_set(delay);
+#else /* !I-pipe core */
 /* With head-optimization, callers are expected to have switched off
    hard-IRQs already -- no need for additional protection in this
    case. */
@@ -89,10 +102,11 @@ static inline void rthal_timer_program_shot(unsigned long delay)
 		apic_write(APIC_TMICT,delay);
 	else
 		/* Pend the timer interrupt. */
-		rthal_schedule_irq_head(RTHAL_APIC_TIMER_IPI);
+		rthal_schedule_irq_head(RTHAL_TIMER_IPI);
 #ifndef CONFIG_XENO_OPT_PIPELINE_HEAD
 	rthal_local_irq_restore_hw(flags);
 #endif /* CONFIG_XENO_OPT_PIPELINE_HEAD */
+#endif /* !I-pipe core */
 }
 
 static const char *const rthal_fault_labels[] = {
