@@ -130,8 +130,6 @@ int __xnpod_set_thread_schedparam(struct xnthread *thread,
 				  const union xnsched_policy_param *sched_param,
 				  int propagate);
 
-void __xnpod_reset_thread(struct xnthread *thread);
-
 #ifdef CONFIG_XENO_HW_FPU
 void xnpod_switch_fpu(xnsched_t *sched);
 #endif /* CONFIG_XENO_HW_FPU */
@@ -279,20 +277,61 @@ static inline void xnpod_schedule(void)
 	 * unlocked context switch.
 	 */
 #if XENO_DEBUG(NUCLEUS)
-	if (testbits(sched->status | sched->lflags, XNKCOUT|XNINIRQ|XNSWLOCK))
+	if (testbits(sched->status | sched->lflags,
+		     XNKCOUT|XNINIRQ|XNINSW|XNINLOCK))
 		return;
 #else /* !XENO_DEBUG(NUCLEUS) */
 	if (testbits(sched->status | sched->lflags,
-		     XNKCOUT|XNINIRQ|XNSWLOCK|XNRESCHED) != XNRESCHED)
+		     XNKCOUT|XNINIRQ|XNINSW|XNRESCHED|XNINLOCK) != XNRESCHED)
 		return;
 #endif /* !XENO_DEBUG(NUCLEUS) */
 
 	__xnpod_schedule(sched);
 }
 
-void xnpod_lock_sched(void);
+void ___xnpod_lock_sched(xnsched_t *sched);
 
-void xnpod_unlock_sched(void);
+void ___xnpod_unlock_sched(xnsched_t *sched);
+
+static inline void __xnpod_lock_sched(void)
+{
+	xnsched_t *sched;
+
+	barrier();
+	sched = xnpod_current_sched();
+	___xnpod_lock_sched(sched);
+}
+
+static inline void __xnpod_unlock_sched(void)
+{
+	xnsched_t *sched;
+
+	barrier();
+	sched = xnpod_current_sched();
+	___xnpod_unlock_sched(sched);
+}
+
+static inline void xnpod_lock_sched(void)
+{
+	xnsched_t *sched;
+	spl_t s;
+
+	xnlock_get_irqsave(&nklock, s);
+	sched = xnpod_current_sched();
+	___xnpod_lock_sched(sched);
+	xnlock_put_irqrestore(&nklock, s);
+}
+
+static inline void xnpod_unlock_sched(void)
+{
+	xnsched_t *sched;
+	spl_t s;
+
+	xnlock_get_irqsave(&nklock, s);
+	sched = xnpod_current_sched();
+	___xnpod_unlock_sched(sched);
+	xnlock_put_irqrestore(&nklock, s);
+}
 
 void xnpod_fire_callouts(xnqueue_t *hookq,
 			 xnthread_t *thread);
