@@ -14,9 +14,6 @@ typedef struct xnshadow_ppd_key {
 typedef struct xnshadow_ppd_t {
     xnshadow_ppd_key_t key;
     xnholder_t link;
-#ifdef XNARCH_HAVE_MAYDAY
-    unsigned long mayday_addr;
-#endif
 #define link2ppd(ln)	container_of(ln, xnshadow_ppd_t, link)
 } xnshadow_ppd_t;
 
