@@ -42,24 +42,17 @@ struct pt_regs;
 struct timespec;
 struct timeval;
 struct xntbase;
-union xnsiginfo;
 
 struct xnskin_props {
 	const char *name;
 	unsigned magic;
 	int nrcalls;
 	void *(*eventcb)(int, void *);
-	int (*sig_unqueue)(struct xnthread *thread, union xnsiginfo __user *si);
 	xnsysent_t *systab;
 	struct xntbase **timebasep;
 	struct module *module;
 };
 
-struct xnskin_slot {
-	struct xnskin_props *props;
-	atomic_counter_t refcnt;
-};
-
 int xnshadow_mount(void);
 
 void xnshadow_cleanup(void);
@@ -70,7 +63,7 @@ void xnshadow_release_events(void);
 
 int xnshadow_map(struct xnthread *thread,
 		 xncompletion_t __user *u_completion,
-		 unsigned long __user *u_mode);
+		 unsigned long __user *u_mode_offset);
 
 void xnshadow_unmap(struct xnthread *thread);
 
@@ -104,12 +97,6 @@ void xnshadow_send_sig(struct xnthread *thread,
 
 void xnshadow_rpi_check(void);
 
-extern struct xnskin_slot muxtable[];
-
-int xnshadow_mark_sig(struct xnthread *thread, unsigned muxid);
-
-void xnshadow_clear_sig(struct xnthread *thread, unsigned muxid);
-
 #ifdef RTHAL_HAVE_RETURN_EVENT
 #define XNARCH_HAVE_MAYDAY  1
 void xnshadow_call_mayday(struct xnthread *thread, int sigtype);
@@ -127,16 +114,12 @@ static inline void xnshadow_call_mayday(struct xnthread *thread,
 
 #endif /* CONFIG_XENO_OPT_PERVASIVE */
 
-#if defined(CONFIG_XENO_OPT_PERVASIVE) && defined(CONFIG_PROC_FS)
+#if defined(CONFIG_XENO_OPT_PERVASIVE) && defined(CONFIG_XENO_OPT_VFILE)
 void xnshadow_init_proc(void);
 void xnshadow_cleanup_proc(void);
-void xnshadow_declare_proc(struct xnskin_slot *iface);
-void xnshadow_remove_proc(const char *iface);
 #else
 static inline void xnshadow_init_proc(void) { }
 static inline void xnshadow_cleanup_proc(void) { }
-#define xnshadow_declare_proc(iface)	do { (void)iface; } while(0)
-#define xnshadow_remove_proc(iface)	do { (void)name; } while(0)
-#endif /* CONFIG_XENO_OPT_PERVASIVE && CONFIG_PROC_FS */
+#endif /* CONFIG_XENO_OPT_PERVASIVE && CONFIG_XENO_OPT_VFILE */
 
 #endif /* !_XENO_NUCLEUS_SHADOW_H */
