@@ -2,9 +2,10 @@
 
 #include_next <stdio.h>
 
-#ifndef STDIO_H
-#define STDIO_H
+#ifndef _XENO_POSIX_STDIO_H
+#define _XENO_POSIX_STDIO_H
 
+#include <stddef.h>
 #include <stdarg.h>
 
 #ifdef __cplusplus
@@ -21,10 +22,18 @@ int __real_printf(const char *fmt, ...);
 
 int __real_puts(const char *s);
 
+int __real_fputs(const char *s, FILE *stream);
+
+int __real_fputc(int c, FILE *stream);
+
+int __real_putchar(int c);
+
+size_t __real_fwrite(const void *ptr, size_t sz, size_t nmemb, FILE *stream);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* STDIO_H */
+#endif /* _XENO_POSIX_STDIO_H */
 
 #endif /* !__KERNEL__ */
