@@ -40,7 +40,6 @@
 #else
 #include <asm/hardirq.h>
 #endif
-#include <asm/system.h>
 #include <asm/irq.h>
 #include <asm/xenomai/hal.h>
 #include <stdarg.h>
@@ -56,6 +55,9 @@ module_param_named(timerfreq, rthal_timerfreq_arg, ulong, 0444);
 unsigned long rthal_clockfreq_arg;
 module_param_named(clockfreq, rthal_clockfreq_arg, ulong, 0444);
 
+unsigned long rthal_disable;
+module_param_named(disable, rthal_disable, ulong, 0444);
+
 #ifdef CONFIG_SMP
 static unsigned long supported_cpus_arg = -1;
 module_param_named(supported_cpus, supported_cpus_arg, ulong, 0444);
@@ -493,32 +495,74 @@ void rthal_apc_free(int apc)
 	clear_bit(apc, &rthal_apc_map);
 }
 
+#ifdef CONFIG_PREEMPT_RT
+
+static inline int setup_apc_handler(void)
+{
+	int cpu;
+
+	for_each_online_cpu(cpu) {
+		rthal_apc_servers[cpu] =
+			kthread_create(rthal_apc_thread,
+				       (void *)(unsigned long)cpu,
+				       "apc/%d", cpu);
+		if (rthal_apc_servers[cpu] == NULL)
+			return -ENOMEM;
+		wake_up_process(rthal_apc_servers[cpu]);
+	}
+
+	return 0;
+}
+
+static inline void cleanup_apc_handler(void)
+{
+	int cpu;
+
+	for_each_online_cpu(cpu) {
+		if (rthal_apc_servers[cpu])
+			kthread_stop(rthal_apc_servers[cpu]);
+	}
+}
+
+#else  /* !CONFIG_PREEMPT_RT */
+
+static inline int setup_apc_handler(void)
+{
+	return 0;
+}
+
+static inline void cleanup_apc_handler(void) { }
+
+#endif  /* !CONFIG_PREEMPT_RT */
+
 int rthal_init(void)
 {
     int err;
+#ifdef CONFIG_SMP
+    int cpu;
+    cpus_clear(rthal_supported_cpus);
+    for (cpu = 0; cpu < num_online_cpus(); cpu++)
+	    if (supported_cpus_arg & (1ul << cpu))
+		    cpu_set(cpu, rthal_supported_cpus);
+#endif /* CONFIG_SMP */
+
+    if (rthal_disable) {
+	    printk("Xenomai: disabled on kernel command line\n");
+	    return -ENOENT;
+    }
 
     err = rthal_arch_init();
 
     if (err)
 	goto out;
 
-#ifdef CONFIG_SMP
-    {
-	int cpu;
-	cpus_clear(rthal_supported_cpus);
-	for (cpu = 0; cpu < BITS_PER_LONG; cpu++)
-	    if (supported_cpus_arg & (1 << cpu))
-		cpu_set(cpu, rthal_supported_cpus);
-    }
-#endif /* CONFIG_SMP */
-
     /*
      * The arch-dependent support must have updated the various
      * frequency args as required.
      */
 
     /* check the CPU frequency first and abort if it's invalid */
-    if (rthal_cpufreq_arg == 0) {
+    if (rthal_clockfreq_arg == 0) {
 	printk(KERN_ERR "Xenomai has detected a CPU frequency of 0. Aborting.\n");
 	return -ENODEV;
     }
@@ -541,13 +585,17 @@ int rthal_init(void)
 
     err = rthal_virtualize_irq(rthal_current_domain,
 			       rthal_apc_virq,
-			       &rthal_apc_handler,
+			       rthal_apc_trampoline,
 			       NULL, NULL, IPIPE_HANDLE_MASK);
     if (err) {
 	printk(KERN_ERR "Xenomai: Failed to virtualize IRQ.\n");
 	goto out_free_irq;
     }
 
+    err = setup_apc_handler();
+    if (err)
+	    goto fail;
+
     err = rthal_register_domain(&rthal_domain,
 				"Xenomai",
 				RTHAL_DOMAIN_ID,
@@ -575,6 +623,8 @@ int rthal_init(void)
     rthal_virtualize_irq(rthal_current_domain, rthal_apc_virq, NULL, NULL, NULL,
 			 0);
 
+    cleanup_apc_handler();
+
   out_free_irq:
     rthal_free_virq(rthal_apc_virq);
 
@@ -593,6 +643,8 @@ void rthal_exit(void)
 	rthal_free_virq(rthal_apc_virq);
     }
 
+    cleanup_apc_handler();
+
     if (rthal_init_done)
 	rthal_unregister_domain(&rthal_domain);
 
