@@ -44,7 +44,6 @@
 #include <linux/module.h>
 #include <linux/console.h>
 #include <linux/bitops.h>
-#include <asm/system.h>
 #include <asm/hardirq.h>
 #include <asm/desc.h>
 #include <asm/io.h>
@@ -68,24 +67,26 @@ static void dummy_mksound(unsigned int hz, unsigned int ticks)
 #include <asm/nmi.h>
 #endif
 
-#ifdef CONFIG_X86_LOCAL_APIC
+#if defined(CONFIG_X86_LOCAL_APIC) || defined(CONFIG_IPIPE_CORE)
 
 unsigned long rthal_timer_calibrate(void)
 {
-	unsigned long flags, v;
+	unsigned long v, flags;
 	rthal_time_t t, dt;
 	int i;
 
+	v = RTHAL_COMPAT_TIMERFREQ / HZ;
+
 	flags = rthal_critical_enter(NULL);
 
+	rthal_timer_program_shot(v);
+
 	t = rthal_rdtsc();
 
-	for (i = 0; i < 20; i++) {
-		v = apic_read(APIC_TMICT);
-		apic_write(APIC_TMICT, v);
-	}
+	for (i = 0; i < 100; i++)
+		rthal_timer_program_shot(v);
 
-	dt = (rthal_rdtsc() - t) / 2;
+	dt = (rthal_rdtsc() - t);
 
 	rthal_critical_exit(flags);
 
@@ -94,7 +95,7 @@ unsigned long rthal_timer_calibrate(void)
 	rthal_trace_max_reset();
 #endif /* CONFIG_IPIPE_TRACE_IRQSOFF */
 
-	return rthal_imuldiv(dt, 20, RTHAL_CPU_FREQ);
+	return rthal_ulldiv(dt, i + 5, NULL);
 }
 
 #else /* !CONFIG_X86_LOCAL_APIC */
@@ -138,7 +139,7 @@ unsigned long rthal_timer_calibrate(void)
 	rthal_trace_max_reset();
 #endif /* CONFIG_IPIPE_TRACE_IRQSOFF */
 
-	return rthal_imuldiv(dt, 20, RTHAL_CPU_FREQ);
+	return rthal_ulldiv(dt, i, NULL);
 }
 
 static void rthal_timer_set_oneshot(void)
@@ -277,7 +278,7 @@ void rthal_timer_release(int cpu)
 
 #endif /* !CONFIG_X86_LOCAL_APIC */
 
-#ifndef CONFIG_X86_TSC
+#if !defined(CONFIG_X86_TSC) && IPIPE_CORE_APIREV < 2
 
 static rthal_time_t rthal_tsc_8254;
 
@@ -290,6 +291,12 @@ void rthal_setup_8254_tsc(void)
 	unsigned long flags;
 	int count;
 
+#ifdef CONFIG_IPIPE_CORE
+	if (cpu_has_tsc)
+		printk("Xenomai: your configuration has tsc disabled while "
+		       "your cpu has a tsc\nYou would get better performances "
+		       "by enabling tsc in kernel configuration.\n");
+#endif
 	rthal_local_irq_save_hw(flags);
 
 	outb_p(0x0, PIT_MODE);
@@ -326,10 +333,15 @@ rthal_time_t rthal_get_8254_tsc(void)
 	return t;
 }
 
-#endif /* !CONFIG_X86_TSC */
+#endif /* !CONFIG_X86_TSC && IPIPE_CORE_APIREV < 2*/
 
 int rthal_arch_init(void)
 {
+#ifdef CONFIG_IPIPE_CORE
+	int rc = wrap_select_timers(&rthal_supported_cpus);
+	if (rc < 0)
+		return rc;
+#else /* !I-pipe core */
 #ifdef CONFIG_X86_LOCAL_APIC
 	if (!boot_cpu_has(X86_FEATURE_APIC)) {
 		printk("Xenomai: Local APIC absent or disabled!\n"
@@ -344,9 +356,19 @@ int rthal_arch_init(void)
 	old_mksound = kd_mksound;
 	kd_mksound = &dummy_mksound;
 #endif /* !CONFIG_X86_LOCAL_APIC && Linux < 2.6 && !CONFIG_X86_TSC && CONFIG_VT */
+#ifdef CONFIG_X86_TSC
+	if (!cpu_has_tsc) {
+		printk("Xenomai: compiled for TSC, but CPU has no TSC\n"
+		       "         Recompile the kernel selecting a CPU without "
+		       "TSC\n");
+		rthal_smi_restore();
+		return -ENODEV;
+	}
+#endif /* X86_TSC */
+#endif /* !I-pipe core */
 
 	if (rthal_cpufreq_arg == 0)
-#ifdef CONFIG_X86_TSC
+#if defined(CONFIG_X86_TSC) || IPIPE_CORE_APIREV >= 2
 		/* FIXME: 4Ghz barrier is close... */
 		rthal_cpufreq_arg = rthal_get_cpufreq();
 #else /* ! CONFIG_X86_TSC */
@@ -358,15 +380,24 @@ int rthal_arch_init(void)
 	if (rthal_clockfreq_arg == 0)
 		rthal_clockfreq_arg = rthal_get_clockfreq();
 
+#ifdef CONFIG_IPIPE_CORE
+	if (rthal_timerfreq_arg == 0)
+		rthal_timerfreq_arg = rthal_get_timerfreq();
+#endif
+
 	return 0;
 }
 
 void rthal_arch_cleanup(void)
 {
+#ifdef CONFIG_IPIPE_CORE
+	ipipe_timers_release();
+#else /* !I-pipe core */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) && !defined(CONFIG_X86_TSC) && defined(CONFIG_VT)
 	/* Restore previous PC speaker code. */
 	kd_mksound = old_mksound;
 #endif /* Linux < 2.6 && !CONFIG_X86_TSC && CONFIG_VT */
+#endif /* !I-pipe core */
 	printk(KERN_INFO "Xenomai: hal/i386 stopped.\n");
 }
 
@@ -374,6 +405,6 @@ void rthal_arch_cleanup(void)
 
 EXPORT_SYMBOL_GPL(rthal_arch_init);
 EXPORT_SYMBOL_GPL(rthal_arch_cleanup);
-#ifndef CONFIG_X86_TSC
+#if !defined(CONFIG_X86_TSC) && IPIPE_CORE_APIREV < 2
 EXPORT_SYMBOL_GPL(rthal_get_8254_tsc);
-#endif /* !CONFIG_X86_TSC */
+#endif /* !CONFIG_X86_TSC && IPIPE_CORE_APIREV < 2 */
