@@ -37,7 +37,6 @@
 #include <linux/module.h>
 #include <linux/console.h>
 #include <linux/bitops.h>
-#include <asm/system.h>
 #include <asm/hardirq.h>
 #include <asm/desc.h>
 #include <asm/io.h>
@@ -52,16 +51,18 @@ unsigned long rthal_timer_calibrate(void)
 	rthal_time_t t, dt;
 	int i;
 
+	v = RTHAL_COMPAT_TIMERFREQ / HZ;
+
 	flags = rthal_critical_enter(NULL);
 
+	rthal_timer_program_shot(v);
+
 	t = rthal_rdtsc();
 
-	for (i = 0; i < 20; i++) {
-		v = apic_read(APIC_TMICT);
-		apic_write(APIC_TMICT, v);
-	}
+	for (i = 0; i < 100; i++)
+		rthal_timer_program_shot(v);
 
-	dt = (rthal_rdtsc() - t) / 2;
+	dt = (rthal_rdtsc() - t);
 
 	rthal_critical_exit(flags);
 
@@ -70,11 +71,17 @@ unsigned long rthal_timer_calibrate(void)
 	rthal_trace_max_reset();
 #endif /* CONFIG_IPIPE_TRACE_IRQSOFF */
 
-	return rthal_imuldiv(dt, 20, RTHAL_CPU_FREQ);
+	return rthal_ulldiv(dt, i + 5, NULL);
 }
 
 int rthal_arch_init(void)
 {
+#ifdef CONFIG_IPIPE_CORE
+	int rc = wrap_select_timers(&rthal_supported_cpus);
+	if (rc < 0)
+		return rc;
+#endif /* I-pipe core */
+
 	if (rthal_cpufreq_arg == 0)
 		/* FIXME: 4Ghz barrier is close... */
 		rthal_cpufreq_arg = rthal_get_cpufreq();
@@ -82,11 +89,19 @@ int rthal_arch_init(void)
 	if (rthal_clockfreq_arg == 0)
 		rthal_clockfreq_arg = rthal_get_clockfreq();
 
+#ifdef CONFIG_IPIPE_CORE
+	if (rthal_timerfreq_arg == 0)
+		rthal_timerfreq_arg = rthal_get_timerfreq();
+#endif
+
 	return 0;
 }
 
 void rthal_arch_cleanup(void)
 {
+#ifdef CONFIG_IPIPE_CORE
+	ipipe_timers_release();
+#endif /* I-pipe core */
 	printk(KERN_INFO "Xenomai: hal/x86_64 stopped.\n");
 }
 
