@@ -168,7 +168,7 @@ static a4l_rngtab_t rng_ni_M_628x_ao = { 8, {
 	RANGE(4, 6),
 	RANGE_ext(-1, 1)
 }};
-static a4l_rngdesc_t range_ni_M_628x_ao = 
+static a4l_rngdesc_t range_ni_M_628x_ao =
 	RNG_GLOBAL(rng_ni_M_628x_ao);
 #endif
 
@@ -177,13 +177,13 @@ static a4l_rngtab_t rng_ni_M_625x_ao = { 3, {
 	RANGE(-5, 5),
 	RANGE_ext(-1, 1)
 }};
-static a4l_rngdesc_t range_ni_M_625x_ao = 
+static a4l_rngdesc_t range_ni_M_625x_ao =
 	RNG_GLOBAL(rng_ni_M_625x_ao);
 
 static a4l_rngtab_t rng_ni_M_622x_ao = { 1, {
 	RANGE(-10, 10),
 }};
-static a4l_rngdesc_t range_ni_M_622x_ao = 
+static a4l_rngdesc_t range_ni_M_622x_ao =
 	RNG_GLOBAL(rng_ni_M_622x_ao);
 
 static ni_board ni_boards[]={
@@ -198,7 +198,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  0,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_speed:	50000,
 		.num_p0_dio_channels = 8,
@@ -216,7 +216,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	10000,
 		.num_p0_dio_channels = 8,
@@ -234,7 +234,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		ao_fifo_depth:  0,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_speed:	100000,
 		.num_p0_dio_channels = 8,
@@ -252,7 +252,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	10000,
 		.num_p0_dio_channels = 8,
@@ -271,7 +271,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
@@ -291,7 +291,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  512,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
@@ -309,7 +309,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  512,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
@@ -328,7 +328,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	10000,
 		.num_p0_dio_channels = 8,
@@ -378,7 +378,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
@@ -411,7 +411,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  0,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_speed:	100000,
 		.num_p0_dio_channels = 8,
@@ -429,7 +429,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  0,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_speed:	100000,
 		.num_p0_dio_channels = 8,
@@ -447,7 +447,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  0,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	100000,
 		.num_p0_dio_channels = 8,
@@ -482,7 +482,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  0,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_speed:	100000,
 		.num_p0_dio_channels = 8,
@@ -501,7 +501,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    1,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_speed:	3000,
 		.num_p0_dio_channels = 8,
 		caldac:         {ad8804_debug,ad8804_debug,ad8522}, /* manual is wrong */
@@ -517,7 +517,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		reg_type:	ni_reg_611x,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
 		ao_speed:	250,
@@ -535,7 +535,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		reg_type:	ni_reg_611x,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_fifo_depth:  2048,
 		ao_speed:	250,
@@ -590,7 +590,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384, /* data sheet says 8192, but fifo really holds 16384 samples */
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
@@ -603,7 +603,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
@@ -616,7 +616,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
@@ -629,7 +629,7 @@ static ni_board ni_boards[]={
 		aobits:         12,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
@@ -642,7 +642,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    0,
 		ao_fifo_depth:  8192,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
@@ -656,7 +656,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    0,
 		ao_fifo_depth:  8192,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6711,
 		caldac:         {ad8804_debug},
@@ -669,7 +669,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
@@ -682,7 +682,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    0,
 		ao_fifo_depth:  16384,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
 		reg_type:	ni_reg_6713,
@@ -699,7 +699,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
@@ -717,7 +717,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         12,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	1000,
 		.num_p0_dio_channels = 8,
@@ -736,7 +736,7 @@ static ni_board ni_boards[]={
 		aobits:         16,
 		ao_unipolar:    1,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_speed:	3000,
 		.num_p0_dio_channels = 8,
 		caldac:         {mb88341,mb88341,ad8522},
@@ -752,7 +752,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		ao_fifo_depth:  2048,
-		.ao_range_table = &range_ni_E_ao_ext,
+		.ao_range_table = &a4l_range_ni_E_ao_ext,
 		ao_unipolar:    1,
 		ao_speed:	10000,
 		.num_p0_dio_channels = 8,
@@ -769,7 +769,7 @@ static ni_board ni_boards[]={
 		n_aochan:       2,
 		aobits:         16,
 		ao_fifo_depth:  0,
-		.ao_range_table = &range_bipolar10,
+		.ao_range_table = &a4l_range_bipolar10,
 		ao_unipolar:    0,
 		ao_speed:	100000,
 		.num_p0_dio_channels = 8,
@@ -1286,7 +1286,7 @@ static void m_series_stc_writew(a4l_dev_t *dev, uint16_t data, int reg)
 		by M_Offset_Static_Digital_Output (32 bit) and
 		M_Offset_SCXI_Serial_Data_Out (8 bit) */
 	default:
-		rtdm_printk("%s: bug! unhandled register=0x%x in switch.\n", 
+		rtdm_printk("%s: bug! unhandled register=0x%x in switch.\n",
 			    __FUNCTION__, reg);
 		BUG();
 		return;
@@ -1322,7 +1322,7 @@ static uint16_t m_series_stc_readw(a4l_dev_t *dev, int reg)
 		break;
 	default:
 		rtdm_printk("%s: bug! "
-			    "unhandled register=0x%x in switch.\n", 
+			    "unhandled register=0x%x in switch.\n",
 			    __FUNCTION__, reg);
 		BUG();
 		return 0;
@@ -1365,7 +1365,7 @@ static void m_series_stc_writel(a4l_dev_t *dev, uint32_t data, int reg)
 		offset = M_Offset_G1_Load_B;
 		break;
 	default:
-		rtdm_printk("%s: bug! unhandled register=0x%x in switch.\n", 
+		rtdm_printk("%s: bug! unhandled register=0x%x in switch.\n",
 			    __FUNCTION__, reg);
 		BUG();
 		return;
@@ -1391,7 +1391,7 @@ static uint32_t m_series_stc_readl(a4l_dev_t *dev, int reg)
 		offset = M_Offset_G1_Save;
 		break;
 	default:
-		rtdm_printk("%s: bug! unhandled register=0x%x in switch.\n", 
+		rtdm_printk("%s: bug! unhandled register=0x%x in switch.\n",
 			    __FUNCTION__, reg);
 		BUG();
 		return 0;
@@ -1426,7 +1426,7 @@ static void m_series_init_eeprom_buffer(a4l_dev_t *dev)
 	old_iodwbsr1_bits = readl(devpriv->mite->mite_io_addr + MITE_IODWBSR_1);
 	old_iodwcr1_bits = readl(devpriv->mite->mite_io_addr + MITE_IODWCR_1);
 	writel(0x0, devpriv->mite->mite_io_addr + MITE_IODWBSR);
-	writel(((0x80 | window_size) | devpriv->mite->daq_phys_addr), 
+	writel(((0x80 | window_size) | devpriv->mite->daq_phys_addr),
 	       devpriv->mite->mite_io_addr + MITE_IODWBSR_1);
 	writel(0x0, devpriv->mite->mite_io_addr + MITE_IODWCR_1);
 	writel(0xf, devpriv->mite->mite_io_addr + 0x30);
@@ -1454,14 +1454,14 @@ static void init_6143(a4l_dev_t *dev)
 	/* Set EOCMode, ADCMode and pipelinedelay */
 	ni_writeb(0x80, PipelineDelay_6143);
 	/* Set EOC Delay */
-	ni_writeb(0x00, EOC_Set_6143);		
+	ni_writeb(0x00, EOC_Set_6143);
 
 	/* Set the FIFO half full level */
-	ni_writel(boardtype.ai_fifo_depth / 2, AIFIFO_Flag_6143);	
+	ni_writel(boardtype.ai_fifo_depth / 2, AIFIFO_Flag_6143);
 
 	/* Strobe Relay disable bit */
 	devpriv->ai_calib_source_enabled = 0;
-	ni_writew(devpriv->ai_calib_source | Calibration_Channel_6143_RelayOff, 
+	ni_writew(devpriv->ai_calib_source | Calibration_Channel_6143_RelayOff,
 		  Calibration_Channel_6143);
 	ni_writew(devpriv->ai_calib_source, Calibration_Channel_6143);
 }
@@ -1482,7 +1482,7 @@ static int pcimio_attach(a4l_dev_t *dev, a4l_lnkdesc_t *arg)
 	}
 
 	for(i = 0; i < n_pcimio_boards && mite == NULL; i++) {
-		mite = mite_find_device(bus, slot, ni_boards[i].device_id);
+		mite = a4l_mite_find_device(bus, slot, ni_boards[i].device_id);
 		board = &ni_boards[i];
 	}
 
@@ -1524,7 +1524,7 @@ static int pcimio_attach(a4l_dev_t *dev, a4l_lnkdesc_t *arg)
 		devpriv->stc_readl = &win_in2;
 	}
 
-	ret = mite_setup(devpriv->mite, 0);
+	ret = a4l_mite_setup(devpriv->mite, 0);
 	if(ret < 0)
 	{
 		a4l_err(dev, "pcmio_attach: error setting up mite\n");
@@ -1543,13 +1543,13 @@ static int pcimio_attach(a4l_dev_t *dev, a4l_lnkdesc_t *arg)
 	}else{
 		a4l_info(dev, "pcimio_attach: found irq %u\n", irq);
 		ret = a4l_request_irq(dev,
-					 irq,
-					 ni_E_interrupt, A4L_IRQ_SHARED, dev);
+				      irq,
+				      a4l_ni_E_interrupt, A4L_IRQ_SHARED, dev);
 		if(ret < 0)
 			a4l_err(dev, "pcimio_attach: irq not available\n");
 	}
 
-	ret = ni_E_init(dev);
+	ret = a4l_ni_E_init(dev);
 	if(ret < 0)
 		return ret;
 
@@ -1568,7 +1568,7 @@ static int pcimio_detach(a4l_dev_t *dev)
 		mite_free_ring(devpriv->ao_mite_ring);
 		mite_free_ring(devpriv->gpct_mite_ring[0]);
 		mite_free_ring(devpriv->gpct_mite_ring[1]);
-		mite_unsetup(devpriv->mite);
+		a4l_mite_unsetup(devpriv->mite);
 	}
 
 	return 0;
