@@ -893,6 +893,20 @@ void xnintr_affinity(xnintr_t *intr, xnarch_cpumask_t cpumask)
 }
 EXPORT_SYMBOL_GPL(xnintr_affinity);
 
+#ifdef CONFIG_XENO_OPT_VFILE
+
+#include <nucleus/vfile.h>
+
+static int xnintr_is_timer_irq(int irq)
+{
+	int cpu;
+
+	for_each_online_cpu(cpu)
+		if (irq == XNARCH_PERCPU_TIMER_IRQ(cpu))
+			return 1;
+	return 0;
+}
+
 #ifdef CONFIG_XENO_OPT_STATS
 int xnintr_query_init(xnintr_iterator_t *iterator)
 {
@@ -934,7 +948,7 @@ int xnintr_query_next(int irq, xnintr_iterator_t *iterator, char *name_buf)
 	}
 
 	if (!iterator->prev) {
-		if (irq == XNARCH_TIMER_IRQ)
+		if (xnintr_is_timer_irq(irq))
 			intr = &nkclock;
 		else
 			intr = xnintr_shirq_first(irq);
@@ -975,28 +989,32 @@ int xnintr_query_next(int irq, xnintr_iterator_t *iterator, char *name_buf)
 }
 #endif /* CONFIG_XENO_OPT_STATS */
 
-#ifdef CONFIG_XENO_OPT_VFILE
-
-#include <nucleus/vfile.h>
-
 static inline int format_irq_proc(unsigned int irq,
 				  struct xnvfile_regular_iterator *it)
 {
 	struct xnintr *intr;
 	spl_t s;
 
-	if (irq == XNARCH_TIMER_IRQ) {
+	if (xnintr_is_timer_irq(irq)) {
 		xnvfile_puts(it, "         [timer]");
 		return 0;
+	}
+
 #ifdef CONFIG_SMP
-	} else if (irq >= RTHAL_SERVICE_IPI0 && irq <= RTHAL_SERVICE_IPI3) {
-		xnvfile_printf(it, "         [IPI%d]", irq - RTHAL_SERVICE_IPI0);
+	if (irq == RTHAL_TIMER_IPI) {
+		xnvfile_puts(it, "         [timer-ipi]");
 		return 0;
-	} else if (irq == RTHAL_CRITICAL_IPI) {
-		xnvfile_puts(it, "         [critical sync]");
+	}
+	if (irq == RTHAL_RESCHEDULE_IPI) {
+		xnvfile_puts(it, "         [reschedule]");
 		return 0;
+	}
+	if (irq == RTHAL_CRITICAL_IPI) {
+		xnvfile_puts(it, "         [sync]");
+		return 0;
+	}
 #endif /* CONFIG_SMP */
-	} else 	if (rthal_virtual_irq_p(irq)) {
+	if (rthal_virtual_irq_p(irq)) {
 		xnvfile_puts(it, "         [virtual]");
 		return 0;
 	}
@@ -1069,7 +1087,7 @@ static int affinity_vfile_show(struct xnvfile_regular_iterator *it,
 
 	for (cpu = 0; cpu < BITS_PER_LONG; cpu++)
 		if (xnarch_cpu_isset(cpu, nkaffinity))
-			val |= (1 << cpu);
+			val |= (1ul << cpu);
 
 	xnvfile_printf(it, "%08lx\n", val);
 
