@@ -814,13 +814,13 @@ EXPORT_SYMBOL_GPL(xnpod_start_thread);
 
 /*!
  * @internal
- * \fn void __xnpod_reset_thread(struct xnthread *thread, int unlock)
+ * \fn void xnpod_reset_thread(struct xnthread *thread, int unlock)
  * \brief Reset the thread.
  *
  * This internal routine resets the state of a thread so that it can
  * be subsequently stopped or restarted.
  */
-void __xnpod_reset_thread(struct xnthread *thread)
+static void xnpod_reset_thread(struct xnthread *thread)
 {
 	/* Break the thread out of any wait it is currently in. */
 	xnpod_unblock_thread(thread);
@@ -885,7 +885,7 @@ void xnpod_stop_thread(struct xnthread *thread)
 
 	xnlock_get_irqsave(&nklock, s);
 	if (!xnthread_test_state(thread, XNDORMANT)) {
-		__xnpod_reset_thread(thread);
+		xnpod_reset_thread(thread);
 		xnpod_suspend_thread(thread, XNDORMANT,
 				     XN_INFINITE, XN_RELATIVE, NULL);
 	} /* Otherwise, it is a nop. */
@@ -940,7 +940,7 @@ void xnpod_restart_thread(xnthread_t *thread)
 
 	xnlock_get_irqsave(&nklock, s);
 
-	__xnpod_reset_thread(thread);
+	xnpod_reset_thread(thread);
 
 	if (thread == xnpod_current_sched()->curr)
 		xnthread_set_state(thread, XNRESTART);
@@ -1187,12 +1187,12 @@ void xnpod_delete_thread(xnthread_t *thread)
 	if (xnthread_test_state(thread, XNPEND))
 		xnsynch_forget_sleeper(thread);
 
+	xnthread_set_state(thread, XNZOMBIE);
+
 	xnsynch_release_all_ownerships(thread);
 
 	__xnpod_giveup_fpu(sched, thread);
 
-	xnthread_set_state(thread, XNZOMBIE);
-
 	if (sched->curr == thread) {
 		/*
 		 * We first need to pick a new curr before
@@ -1200,10 +1200,11 @@ void xnpod_delete_thread(xnthread_t *thread)
 		 * thread zombie state to go through the rescheduling
 		 * procedure then actually destroy the thread object.
 		 */
+		__clrbits(sched->lflags, XNINLOCK);
 		xnsched_set_resched(sched);
 		xnpod_schedule();
 #ifdef CONFIG_XENO_HW_UNLOCKED_SWITCH
-	} else if (!testbits(sched->status, XNSWLOCK) &&
+	} else if (!testbits(sched->status, XNINSW) &&
 		   !xnthread_test_state(thread, XNMIGRATE)) {
 		/*
 		 * When killing a thread in the course of a context
@@ -1399,6 +1400,10 @@ void xnpod_suspend_thread(xnthread_t *thread, xnflags_t mask,
 		 */
 		if ((mask & XNRELAX) == 0 &&
 		    xnthread_test_info(thread, XNKICKED)) {
+			if (wchan) {
+				thread->wchan = wchan;
+				xnsynch_forget_sleeper(thread);
+			}
 			xnthread_clear_info(thread, XNRMID | XNTIMEO);
 			xnthread_set_info(thread, XNBREAK);
 			goto unlock_and_exit;
@@ -1448,6 +1453,7 @@ void xnpod_suspend_thread(xnthread_t *thread, xnflags_t mask,
 #endif /* __XENO_SIM__ */
 
 	if (thread == sched->curr) {
+		__clrbits(sched->lflags, XNINLOCK);
 		/*
 		 * If the current thread is being relaxed, we must
 		 * have been called from xnshadow_relax(), in which
@@ -2082,7 +2088,7 @@ static inline void xnpod_switch_to(xnsched_t *sched,
 {
 #ifdef CONFIG_XENO_HW_UNLOCKED_SWITCH
 	sched->last = prev;
-	__setbits(sched->status, XNSWLOCK);
+	__setbits(sched->status, XNINSW);
 	xnlock_clear_irqon(&nklock);
 #endif /* !CONFIG_XENO_HW_UNLOCKED_SWITCH */
 
@@ -2298,7 +2304,6 @@ reschedule:
 	xnpod_run_hooks(&nkpod->tswitchq, curr, "SWITCH");
 
       signal_unlock_and_exit:
-
 	if (xnthread_signaled_p(curr))
 		xnpod_dispatch_signals();
 
@@ -2306,6 +2311,9 @@ reschedule:
 	    xnsched_maybe_resched_after_unlocked_switch(sched))
 		goto reschedule;
 
+	if (xnthread_lock_count(curr))
+		__setbits(sched->lflags, XNINLOCK);
+
 	xnlock_put_irqrestore(&nklock, s);
 
 	return;
@@ -2332,40 +2340,31 @@ reschedule:
 }
 EXPORT_SYMBOL_GPL(__xnpod_schedule);
 
-void xnpod_lock_sched(void)
+void ___xnpod_lock_sched(xnsched_t *sched)
 {
-	struct xnthread *curr;
-	spl_t s;
+	struct xnthread *curr = sched->curr;
 
-	xnlock_get_irqsave(&nklock, s);
-
-	curr = xnpod_current_thread();
-
-	if (xnthread_lock_count(curr)++ == 0)
+	if (xnthread_lock_count(curr)++ == 0) {
+		__setbits(sched->lflags, XNINLOCK);
 		xnthread_set_state(curr, XNLOCK);
-
-	xnlock_put_irqrestore(&nklock, s);
+	}
 }
-EXPORT_SYMBOL_GPL(xnpod_lock_sched);
+EXPORT_SYMBOL_GPL(___xnpod_lock_sched);
 
-void xnpod_unlock_sched(void)
+void ___xnpod_unlock_sched(xnsched_t *sched)
 {
-	struct xnthread *curr;
-	spl_t s;
-
-	xnlock_get_irqsave(&nklock, s);
-
-	curr = xnpod_current_thread();
+	struct xnthread *curr = sched->curr;
+	XENO_ASSERT(NUCLEUS, xnthread_lock_count(curr) > 0,
+		    xnpod_fatal("Unbalanced lock/unlock");
+		    );
 
 	if (--xnthread_lock_count(curr) == 0) {
 		xnthread_clear_state(curr, XNLOCK);
-		xnsched_set_self_resched(curr->sched);
+		__clrbits(sched->lflags, XNINLOCK);
 		xnpod_schedule();
 	}
-
-	xnlock_put_irqrestore(&nklock, s);
 }
-EXPORT_SYMBOL_GPL(xnpod_unlock_sched);
+EXPORT_SYMBOL_GPL(___xnpod_unlock_sched);
 
 /*!
  * \fn int xnpod_add_hook(int type,void (*routine)(xnthread_t *))
@@ -3140,8 +3139,33 @@ static int lock_vfile_show(struct xnvfile_regular_iterator *it, void *data)
 	return 0;
 }
 
+static ssize_t lock_vfile_store(struct xnvfile_input *input)
+{
+	ssize_t ret;
+	spl_t s;
+	int cpu;
+
+	long val;
+
+	ret = xnvfile_get_integer(input, &val);
+	if (ret < 0)
+		return ret;
+
+	if (val != 0)
+		return -EINVAL;
+
+	for_each_online_cpu(cpu) {
+		xnlock_get_irqsave(&nklock, s);
+		memset(&xnlock_stats[cpu], '\0', sizeof(xnlock_stats[cpu]));
+		xnlock_put_irqrestore(&nklock, s);
+	}
+
+	return ret;
+}
+
 static struct xnvfile_regular_ops lock_vfile_ops = {
 	.show = lock_vfile_show,
+	.store = lock_vfile_store,
 };
 
 static struct xnvfile_regular lock_vfile = {
