@@ -1,4 +1,4 @@
-/*!\file sched.c
+/*!\file nucleus/sched.c
  * \author Philippe Gerum
  *
  * Copyright (C) 2008 Philippe Gerum <rpm@xenomai.org>.
@@ -51,13 +51,13 @@ static void xnsched_register_class(struct xnsched_class *sched_class)
 void xnsched_register_classes(void)
 {
 	xnsched_register_class(&xnsched_class_idle);
-	xnsched_register_class(&xnsched_class_rt);
-#ifdef CONFIG_XENO_OPT_SCHED_SPORADIC
-	xnsched_register_class(&xnsched_class_sporadic);
-#endif
 #ifdef CONFIG_XENO_OPT_SCHED_TP
 	xnsched_register_class(&xnsched_class_tp);
 #endif
+#ifdef CONFIG_XENO_OPT_SCHED_SPORADIC
+	xnsched_register_class(&xnsched_class_sporadic);
+#endif
+	xnsched_register_class(&xnsched_class_rt);
 }
 
 #ifdef CONFIG_XENO_OPT_WATCHDOG
@@ -349,6 +349,7 @@ struct xnsched *xnsched_finish_unlocked_switch(struct xnsched *sched)
 	spl_t s;
 
 	xnlock_get_irqsave(&nklock, s);
+	(void)s;
 
 #ifdef CONFIG_SMP
 	/* If current thread migrated while suspended */
@@ -356,7 +357,7 @@ struct xnsched *xnsched_finish_unlocked_switch(struct xnsched *sched)
 #endif /* CONFIG_SMP */
 
 	last = sched->last;
-	__clrbits(sched->status, XNSWLOCK);
+	__clrbits(sched->status, XNINSW);
 
 	/* Detect a thread which called xnpod_migrate_thread */
 	if (last->sched != sched) {
@@ -908,10 +909,11 @@ scan_irqs:
 	if (priv->irq >= XNARCH_NR_IRQS)
 		return 0;	/* All done. */
 
-	ret = xnintr_query_next(priv->irq++, &priv->intr_it, p->name);
+	ret = xnintr_query_next(priv->irq, &priv->intr_it, p->name);
 	if (ret) {
 		if (ret == -EAGAIN)
 			xnvfile_touch(it->vfile); /* force rewind. */
+		priv->irq++;
 		return VFILE_SEQ_SKIP;
 	}
 
