@@ -113,7 +113,7 @@ static int nucleus_muxid = -1;
 
 static struct semaphore completion_mutex;
 
-static DEFINE_SEMAPHORE(registration_mutex);
+static DEFINE_BINARY_SEMAPHORE(registration_mutex);
 
 static inline struct task_struct *get_switch_lock_owner(void)
 {
@@ -820,7 +820,7 @@ static void lostage_handler(void *cookie)
 
 static void schedule_linux_call(int type, struct task_struct *p, int arg)
 {
-	int cpu = rthal_processor_id(), reqnum;
+	int cpu, reqnum;
 	struct __lostagerq *rq;
 	spl_t s;
 
@@ -832,6 +832,7 @@ static void schedule_linux_call(int type, struct task_struct *p, int arg)
 
 	splhigh(s);
 
+	cpu = rthal_processor_id();
 	rq = &lostagerq[cpu];
 	reqnum = rq->in;
 	rq->in = (reqnum + 1) & (LO_MAX_REQUESTS - 1);
@@ -1633,9 +1634,7 @@ static unsigned long map_mayday_page(struct task_struct *p)
 
 	old_fops = filp->f_op;
 	filp->f_op = &mayday_fops;
-	down_write(&p->mm->mmap_sem);
-	u_addr = do_mmap(filp, 0, PAGE_SIZE, PROT_EXEC|PROT_READ, MAP_SHARED, 0);
-	up_write(&p->mm->mmap_sem);
+	u_addr = vm_mmap(filp, 0, PAGE_SIZE, PROT_EXEC|PROT_READ, MAP_SHARED, 0);
 	filp->f_op = (typeof(filp->f_op))old_fops;
 	filp_close(filp, p->files);
 
@@ -1663,7 +1662,12 @@ static int xnshadow_sys_mayday(struct pt_regs *regs)
 		 * relax. See do_hisyscall_event().
 		 */
 		xnarch_fixup_mayday(xnthread_archtcb(cur), regs);
-		return 0;
+
+		/* returning 0 here would clobber the register holding
+		   the return value. Instead, return whatever value
+		   xnarch_fixup_mayday set for this register, in order
+		   not to undo what xnarch_fixup_mayday did. */
+		return __xn_reg_rval(regs);
 	}
 
 	printk(KERN_WARNING
@@ -1927,8 +1931,8 @@ void xnshadow_signal_completion(xncompletion_t __user *u_completion, int err)
 {
 	xncompletion_t completion;
 	struct task_struct *p;
-	int discarded;
 	pid_t pid;
+	int ret;
 
 	/* Hold a mutex to avoid missing a wakeup signal. */
 	down(&completion_mutex);
@@ -1940,7 +1944,8 @@ void xnshadow_signal_completion(xncompletion_t __user *u_completion, int err)
 
 	/* Poor man's semaphore V. */
 	completion.syncflag = err ? : completion_value_ok;
-	discarded = __xn_safe_copy_to_user(u_completion, &completion, sizeof(completion));
+	ret = __xn_safe_copy_to_user(u_completion, &completion, sizeof(completion));
+	(void)ret;
 	pid = completion.pid;
 
 	up(&completion_mutex);
@@ -1963,7 +1968,7 @@ static int xnshadow_sys_completion(struct pt_regs *regs)
 {
 	xncompletion_t __user *u_completion;
 	xncompletion_t completion;
-	int discarded;
+	int ret;
 
 	u_completion = (xncompletion_t __user *)__xn_reg_arg1(regs);
 
@@ -1993,8 +1998,8 @@ static int xnshadow_sys_completion(struct pt_regs *regs)
 
 		if (signal_pending(current)) {
 			completion.pid = -1;
-			discarded = __xn_safe_copy_to_user(u_completion, &completion, sizeof(completion));
-			return -ERESTARTSYS;
+			ret = __xn_safe_copy_to_user(u_completion, &completion, sizeof(completion));
+			return ret ? -EFAULT : -ERESTARTSYS;
 		}
 	}
 
@@ -2228,7 +2233,9 @@ void xnshadow_send_sig(xnthread_t *thread, int sig, int arg, int specific)
 }
 EXPORT_SYMBOL_GPL(xnshadow_send_sig);
 
-static inline int do_hisyscall_event(unsigned event, unsigned domid, void *data)
+static inline
+int do_hisyscall_event(unsigned event, rthal_pipeline_stage_t *stage,
+		       void *data)
 {
 	struct pt_regs *regs = (struct pt_regs *)data;
 	int muxid, muxop, switched, err, sigs;
@@ -2307,7 +2314,7 @@ static inline int do_hisyscall_event(unsigned event, unsigned domid, void *data)
 	if ((sysflags & __xn_exec_lostage) != 0) {
 		/* Syscall must run into the Linux domain. */
 
-		if (domid == RTHAL_DOMAIN_ID) {
+		if (stage == &rthal_domain) {
 			/* Request originates from the Xenomai domain: just relax the
 			   caller and execute the syscall immediately after. */
 			xnshadow_relax(1, SIGDEBUG_MIGRATE_SYSCALL);
@@ -2321,7 +2328,7 @@ static inline int do_hisyscall_event(unsigned event, unsigned domid, void *data)
 		/* Syscall must be processed either by Xenomai, or by the
 		   calling domain. */
 
-		if (domid != RTHAL_DOMAIN_ID)
+		if (stage != &rthal_domain)
 			/* Request originates from the Linux domain: propagate the
 			   event to our Linux-based handler, so that the caller is
 			   hardened and the syscall is eventually executed from
@@ -2446,7 +2453,9 @@ static inline int do_hisyscall_event(unsigned event, unsigned domid, void *data)
 
 RTHAL_DECLARE_EVENT(hisyscall_event);
 
-static inline int do_losyscall_event(unsigned event, unsigned domid, void *data)
+static inline
+int do_losyscall_event(unsigned event, rthal_pipeline_stage_t *stage,
+		       void *data)
 {
 	int muxid, muxop, sysflags, switched, err, sigs;
 	struct pt_regs *regs = (struct pt_regs *)data;
@@ -2533,7 +2542,8 @@ static inline int do_losyscall_event(unsigned event, unsigned domid, void *data)
 			   xnthread_get_rescnt(thread) == 0)
 			sysflags |= __xn_exec_switchback;
 	}
-	if (!sigs && (sysflags & __xn_exec_switchback) != 0 && switched)
+	if (!sigs && (sysflags & __xn_exec_switchback) != 0
+	    && (switched || xnpod_primary_p()))
 		xnshadow_relax(0, 0);
 
       ret_handled:
@@ -2590,13 +2600,12 @@ RTHAL_DECLARE_EXIT_EVENT(taskexit_event);
 static inline void do_schedule_event(struct task_struct *next_task)
 {
 	struct task_struct *prev_task;
-	struct xnthread *prev, *next;
+	struct xnthread *next;
 
 	if (!xnpod_active_p())
 		return;
 
 	prev_task = current;
-	prev = xnshadow_thread(prev_task);
 	next = xnshadow_thread(next_task);
 	set_switch_lock_owner(prev_task);
 
