@@ -608,7 +608,8 @@ static void xnsynch_clear_boost(struct xnsynch *synch,
 			target = owner;
 	}
 
-	if (w_cprio(owner) != wprio)
+	if (w_cprio(owner) != wprio &&
+	    !xnthread_test_state(owner, XNZOMBIE))
 		xnsynch_renice_thread(owner, target);
 }
 
@@ -684,6 +685,7 @@ xnsynch_release_thread(struct xnsynch *synch, struct xnthread *lastowner)
 
 	XENO_BUGON(NUCLEUS, !testbits(synch->status, XNSYNCH_OWNER));
 
+#ifdef CONFIG_XENO_OPT_PERVASIVE
 	if (xnthread_test_state(lastowner, XNOTHER)) {
 		if (xnthread_get_rescnt(lastowner) == 0)
 			xnshadow_send_sig(lastowner, SIGDEBUG,
@@ -691,6 +693,7 @@ xnsynch_release_thread(struct xnsynch *synch, struct xnthread *lastowner)
 		else
 			xnthread_dec_rescnt(lastowner);
 	}
+#endif
 	lastownerh = xnthread_handle(lastowner);
 
 	if (use_fastlock &&
@@ -1004,7 +1007,7 @@ EXPORT_SYMBOL_GPL(xnsynch_release_all_ownerships);
 void xnsynch_detect_relaxed_owner(struct xnsynch *synch, struct xnthread *sleeper)
 {
 	if (xnthread_test_state(sleeper, XNTRAPSW) &&
-	    xnthread_test_info(sleeper, XNSWREP) &&
+	    !xnthread_test_info(sleeper, XNSWREP) &&
 	    xnthread_test_state(synch->owner, XNRELAX)) {
 		xnthread_set_info(sleeper, XNSWREP);
 		xnshadow_send_sig(sleeper, SIGDEBUG,
