@@ -52,10 +52,12 @@ int xnthread_init(struct xnthread *thread,
 {
 	unsigned int stacksize = attr->stacksize;
 	xnflags_t flags = attr->flags;
+	struct xnarchtcb *tcb;
 	int ret;
 
 	/* Setup the TCB. */
-	xnarch_init_tcb(xnthread_archtcb(thread));
+	tcb = xnthread_archtcb(thread);
+	xnarch_init_tcb(tcb);
 
 	flags &= ~XNSUSP;
 #ifndef CONFIG_XENO_HW_FPU
@@ -87,13 +89,15 @@ int xnthread_init(struct xnthread *thread,
 	}
 #endif
 #else
-	ret = xnarch_alloc_stack(xnthread_archtcb(thread), stacksize);
+	ret = xnarch_alloc_stack(tcb, stacksize);
 	if (ret) {
 		xnlogerr("%s: no stack for kernel thread '%s' (raise CONFIG_XENO_OPT_SYS_STACKPOOLSZ)\n",
 			 __FUNCTION__, attr->name);
 		return ret;
 	}
 #endif
+	if (stacksize)
+		memset(xnarch_stack_base(tcb), 0, stacksize);
 
 	if (attr->name)
 		xnobject_copy_name(thread->name, attr->name);
@@ -122,6 +126,7 @@ int xnthread_init(struct xnthread *thread,
 	thread->wchan = NULL;
 	thread->wwake = NULL;
 	thread->wcontext = NULL;
+	thread->hrescnt = 0;
 	thread->errcode = 0;
 	thread->registry.handle = XN_NO_HANDLE;
 	thread->registry.waitkey = NULL;
@@ -145,10 +150,6 @@ int xnthread_init(struct xnthread *thread,
 #endif /* CONFIG_XENO_OPT_SELECT */
 	initpq(&thread->claimq);
 
-#ifdef CONFIG_XENO_OPT_PERVASIVE
-	xnthread_set_sigpending(thread, 0);
-#endif /* CONFIG_XENO_OPT_PERVASIVE */
-
 	thread->sched = sched;
 	thread->init_class = sched_class;
 	thread->base_class = NULL; /* xnsched_set_policy() will set it. */
@@ -172,7 +173,7 @@ int xnthread_init(struct xnthread *thread,
 
 fail:
 #if CONFIG_XENO_OPT_SYS_STACKPOOLSZ > 0
-	xnarch_free_stack(xnthread_archtcb(thread));
+	xnarch_free_stack(tcb);
 #endif
 	return ret;
 }
