@@ -2,7 +2,7 @@
  * @file
  * This file is part of the Xenomai project.
  *
- * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org> 
+ * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,77 +45,111 @@
 #include <native/task.h>
 #include <native/event.h>
 
-#ifdef CONFIG_PROC_FS
+#ifdef CONFIG_XENO_OPT_VFILE
 
-static int __event_read_proc(char *page,
-			     char **start,
-			     off_t off, int count, int *eof, void *data)
+struct vfile_priv {
+	struct xnpholder *curr;
+	unsigned long value;
+};
+
+struct vfile_data {
+	int mode;
+	unsigned long mask;
+	char name[XNOBJECT_NAME_LEN];
+};
+
+static int vfile_rewind(struct xnvfile_snapshot_iterator *it)
 {
-	RT_EVENT *event = (RT_EVENT *)data;
-	char *p = page;
-	int len;
-	spl_t s;
+	struct vfile_priv *priv = xnvfile_iterator_priv(it);
+	RT_EVENT *event = xnvfile_priv(it->vfile);
 
-	xnlock_get_irqsave(&nklock, s);
+	event = xeno_h2obj_validate(event, XENO_EVENT_MAGIC, RT_EVENT);
+	if (event == NULL)
+		return -EIDRM;
 
-	p += sprintf(p, "=0x%lx\n", event->value);
+	priv->curr = getheadpq(xnsynch_wait_queue(&event->synch_base));
+	priv->value = event->value;
 
-	if (xnsynch_nsleepers(&event->synch_base) > 0) {
-		xnpholder_t *holder;
-
-		/* Pended event -- dump waiters. */
-
-		holder = getheadpq(xnsynch_wait_queue(&event->synch_base));
-
-		while (holder) {
-			xnthread_t *sleeper = link2thread(holder, plink);
-			RT_TASK *task = thread2rtask(sleeper);
-			const char *mode =
-			    (task->wait_args.event.
-			     mode & EV_ANY) ? "any" : "all";
-			unsigned long mask = task->wait_args.event.mask;
-			p += sprintf(p, "+%s (mask=0x%lx, %s)\n",
-				     xnthread_name(sleeper), mask, mode);
-			holder =
-			    nextpq(xnsynch_wait_queue(&event->synch_base),
-				   holder);
-		}
-	}
-
-	xnlock_put_irqrestore(&nklock, s);
-
-	len = (p - page) - off;
-	if (len <= off + count)
-		*eof = 1;
-	*start = page + off;
-	if (len > count)
-		len = count;
-	if (len < 0)
-		len = 0;
-
-	return len;
+	return xnsynch_nsleepers(&event->synch_base);
 }
 
-extern xnptree_t __native_ptree;
+static int vfile_next(struct xnvfile_snapshot_iterator *it, void *data)
+{
+	struct vfile_priv *priv = xnvfile_iterator_priv(it);
+	RT_EVENT *event = xnvfile_priv(it->vfile);
+	struct vfile_data *p = data;
+	struct xnthread *thread;
+	RT_TASK *task;
 
-static xnpnode_t __event_pnode = {
+	priv->value = event->value; /* Refresh as we collect. */
 
-	.dir = NULL,
-	.type = "events",
-	.entries = 0,
-	.read_proc = &__event_read_proc,
-	.write_proc = NULL,
-	.root = &__native_ptree,
+	if (priv->curr == NULL)
+		return 0;	/* We are done. */
+
+	/* Fetch current waiter, advance list cursor. */
+	thread = link2thread(priv->curr, plink);
+	priv->curr = nextpq(xnsynch_wait_queue(&event->synch_base),
+			    priv->curr);
+
+	/* Collect thread name to be output in ->show(). */
+	strncpy(p->name, xnthread_name(thread), sizeof(p->name));
+	task = thread2rtask(thread);
+	p->mode = task->wait_args.event.mode;
+	p->mask = task->wait_args.event.mask;
+
+	return 1;
+}
+
+static int vfile_show(struct xnvfile_snapshot_iterator *it, void *data)
+{
+	struct vfile_priv *priv = xnvfile_iterator_priv(it);
+	struct vfile_data *p = data;
+
+	if (p == NULL) {	/* Dump header. */
+		/* Always dump current event mask value. */
+		xnvfile_printf(it, "=0x%lx\n", priv->value);
+		if (it->nrdata > 0)
+			xnvfile_printf(it, "\n%10s  %4s  %s\n",
+				       "MASK", "MODE", "WAITER");
+	} else
+		xnvfile_printf(it, "0x%-8lx  %4s  %.*s\n",
+			       p->mask,
+			       p->mode & EV_ANY ? "any" : "all",
+			       (int)sizeof(p->name), p->name);
+
+	return 0;
+}
+
+static struct xnvfile_snapshot_ops vfile_ops = {
+	.rewind = vfile_rewind,
+	.next = vfile_next,
+	.show = vfile_show,
 };
 
-#else /* !CONFIG_PROC_FS */
+extern struct xnptree __native_ptree;
 
-static xnpnode_t __event_pnode = {
-
-	.type = "events"
+static struct xnpnode_snapshot __event_pnode = {
+	.node = {
+		.dirname = "events",
+		.root = &__native_ptree,
+		.ops = &xnregistry_vfsnap_ops,
+	},
+	.vfile = {
+		.privsz = sizeof(struct vfile_priv),
+		.datasz = sizeof(struct vfile_data),
+		.ops = &vfile_ops,
+	},
 };
 
-#endif /* !CONFIG_PROC_FS */
+#else /* !CONFIG_XENO_OPT_VFILE */
+
+static struct xnpnode_snapshot __event_pnode = {
+	.node = {
+		.dirname = "events",
+	},
+};
+
+#endif /* !CONFIG_XENO_OPT_VFILE */
 
 /**
  * @fn int rt_event_create(RT_EVENT *event,const char *name,unsigned long ivalue,int mode)
@@ -205,7 +239,7 @@ int rt_event_create(RT_EVENT *event,
 	 */
 	if (name) {
 		err = xnregistry_enter(event->name, event, &event->handle,
-				       &__event_pnode);
+				       &__event_pnode.node);
 
 		if (err)
 			rt_event_delete(event);
@@ -811,10 +845,10 @@ void __native_event_pkg_cleanup(void)
 
 /*@}*/
 
-EXPORT_SYMBOL(rt_event_create);
-EXPORT_SYMBOL(rt_event_delete);
-EXPORT_SYMBOL(rt_event_signal);
-EXPORT_SYMBOL(rt_event_wait);
-EXPORT_SYMBOL(rt_event_wait_until);
-EXPORT_SYMBOL(rt_event_clear);
-EXPORT_SYMBOL(rt_event_inquire);
+EXPORT_SYMBOL_GPL(rt_event_create);
+EXPORT_SYMBOL_GPL(rt_event_delete);
+EXPORT_SYMBOL_GPL(rt_event_signal);
+EXPORT_SYMBOL_GPL(rt_event_wait);
+EXPORT_SYMBOL_GPL(rt_event_wait_until);
+EXPORT_SYMBOL_GPL(rt_event_clear);
+EXPORT_SYMBOL_GPL(rt_event_inquire);
