@@ -2,7 +2,7 @@
  * @file
  * This file is part of the Xenomai project.
  *
- * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org> 
+ * @note Copyright (C) 2004 Philippe Gerum <rpm@xenomai.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,8 +44,10 @@
 /** @example sigxcpu.c */
 /** @example trivial-periodic.c */
 
+#include <nucleus/sys_ppd.h>
 #include <nucleus/pod.h>
 #include <nucleus/heap.h>
+#include <nucleus/assert.h>
 #include <native/task.h>
 #include <native/timer.h>
 
@@ -77,6 +79,9 @@ static void __task_delete_hook(xnthread_t *thread)
 	   hooks are done. */
 	xnsynch_destroy(&task->mrecv);
 	xnsynch_destroy(&task->msendq);
+#ifdef CONFIG_XENO_FASTSYNCH
+	xnheap_free(&xnsys_ppd_get(0)->sem_heap, task->msendq.fastlock);
+#endif
 #endif /* CONFIG_XENO_OPT_NATIVE_MPS */
 
 	xnsynch_destroy(&task->safesynch);
@@ -249,17 +254,24 @@ void __native_task_pkg_cleanup(void)
 int rt_task_create(RT_TASK *task,
 		   const char *name, int stksize, int prio, int mode)
 {
+#ifdef CONFIG_XENO_OPT_NATIVE_MPS
+	xnarch_atomic_t *fastlock = NULL;
+#endif
 	union xnsched_policy_param param;
 	struct xnthread_init_attr attr;
 	int err = 0, cpumask, cpu;
 	xnflags_t bflags;
 	spl_t s;
 
-	if (prio < T_LOPRIO || prio > T_HIPRIO)
-		return -EINVAL;
+	if (prio < T_LOPRIO || prio > T_HIPRIO) {
+		err = -EINVAL;
+		goto fail;
+	}
 
-	if (xnpod_asynch_p())
-		return -EPERM;
+	if (xnpod_asynch_p()) {
+		err = -EPERM;
+		goto fail;
+	}
 
 	bflags = mode & (XNFPU | XNSHADOW | XNSUSP);
 
@@ -273,10 +285,10 @@ int rt_task_create(RT_TASK *task,
 	attr.stacksize = stksize;
 	param.rt.prio = prio;
 
-	if (xnpod_init_thread(&task->thread_base,
-			      &attr, &xnsched_class_rt, &param) != 0)
-		/* Assume this is the only possible failure. */
-		return -ENOMEM;
+	err = xnpod_init_thread(&task->thread_base, &attr, &xnsched_class_rt,
+				&param);
+	if (err)
+		goto fail;
 
 	inith(&task->link);
 	task->suspend_depth = (bflags & XNSUSP) ? 1 : 0;
@@ -296,9 +308,21 @@ int rt_task_create(RT_TASK *task,
 		task->affinity = XNPOD_ALL_CPUS;
 
 #ifdef CONFIG_XENO_OPT_NATIVE_MPS
+#ifdef CONFIG_XENO_FASTSYNCH
+	/* Allocate lock memory for in-kernel use */
+	fastlock = xnheap_alloc(&xnsys_ppd_get(0)->sem_heap,
+				sizeof(*fastlock));
+	if (fastlock == NULL) {
+		xnpod_delete_thread(&task->thread_base);
+		return -ENOMEM;
+	}
+#endif /* CONFIG_XENO_FASTSYNCH */
 	xnsynch_init(&task->mrecv, XNSYNCH_FIFO, NULL);
-	xnsynch_init(&task->msendq,
-		     XNSYNCH_PRIO | XNSYNCH_PIP | XNSYNCH_OWNER, NULL);
+	/*
+	 * Make the task acquire the synchronization object
+	 * indefinitely, without raising the resource count though.
+	 */
+	xnsynch_init(&task->msendq, XNSYNCH_PIP, fastlock);
 	xnsynch_set_owner(&task->msendq, &task->thread_base);
 	task->flowgen = 0;
 #endif /* CONFIG_XENO_OPT_NATIVE_MPS */
@@ -315,6 +339,16 @@ int rt_task_create(RT_TASK *task,
 	err = xnthread_register(&task->thread_base, name ? task->rname : "");
 	if (err)
 		xnpod_delete_thread(&task->thread_base);
+#if defined(CONFIG_XENO_OPT_NATIVE_MPS) && defined(CONFIG_XENO_FASTSYNCH)
+	else
+		xnsynch_fast_acquire(fastlock, xnthread_handle(&task->thread_base));
+#endif
+
+	return err;
+
+      fail:
+	if (xnthread_test_state(&task->thread_base, XNSHADOW))
+		xnfree(task);
 
 	return err;
 }
@@ -1433,10 +1467,10 @@ int rt_task_notify(RT_TASK *task, rt_sigset_t signals)
  * inheriting the priority of the running shadow Xenomai thread. Use
  * CONFIG_XENO_OPT_RPIOFF to globally disable priority coupling.
  *
- * - T_PRIMARY can be passed to switch the current user-space task to
- * primary mode (setmask |= T_PRIMARY), or secondary mode (clrmask |=
- * T_PRIMARY). Upon return from rt_task_set_mode(), the user-space
- * task will run into the specified domain.
+ * - T_CONFORMING can be passed in @a setmask to switch the current
+ * user-space task to its preferred runtime mode. The only meaningful
+ * use of this switch is to force a real-time shadow back to primary
+ * mode. Any other use either cause to a nop, or an error.
  *
  * Normally, this service can only be called on behalf of a regular
  * real-time task, either running in kernel or user-space. However, as
@@ -1461,7 +1495,8 @@ int rt_task_notify(RT_TASK *task, rt_sigset_t signals)
  * @return 0 is returned upon success, or:
  *
  * - -EINVAL if either @a setmask or @a clrmask specifies invalid
- * bits. T_PRIMARY is invalid for kernel-based tasks.
+ * bits. T_CONFORMING is always invalid in @a clrmask, or when applied
+ * in @a setmask to kernel-based tasks.
  *
  * - -EPERM is returned if this service was not called from a
  * real-time task context.
@@ -1774,6 +1809,16 @@ ssize_t rt_task_send(RT_TASK *task,
 	else
 		client->wait_args.mps.mcb_r.size = 0;
 
+	/*
+	 * We distinguish rt_task_unblock() from rt_task_reply() calls
+	 * on the sender, checking the value of the flowid token on
+	 * return from suspension. If zero, rt_task_unblock() was
+	 * called to release the client prior to receiving any reply,
+	 * otherwise rt_task_reply() was invoked for the current
+	 * transaction.
+	 */
+	client->wait_args.mps.mcb_r.flowid = 0;
+
 	/*
 	 * Wake up the server if it is currently waiting for a
 	 * message, then sleep on the send queue, waiting for the
@@ -1799,11 +1844,16 @@ ssize_t rt_task_send(RT_TASK *task,
 		err = -EIDRM;	/* Receiver deleted while pending. */
 	else if (info & XNTIMEO)
 		err = -ETIMEDOUT;	/* Timeout. */
-	else if (info & XNBREAK)
-		err = -EINTR;	/* Unblocked. */
 	else {
-		rsize = client->wait_args.mps.mcb_r.size;
+		XENO_BUGON(NATIVE, (info & XNBREAK) == 0);
 
+		if (client->wait_args.mps.mcb_r.flowid == 0) {
+			/* Task unblocked prior to being replied to. */
+			err = -EINTR;
+			goto unlock_and_exit;
+		}
+
+		rsize = client->wait_args.mps.mcb_r.size;
 		if (rsize > 0) {
 			/* Ok, the message has been processed and answered by the
 			   remote, and a memory address is available to pass the
@@ -1950,21 +2000,28 @@ int rt_task_receive(RT_TASK_MCB *mcb_r, RTIME timeout)
 	}
 
 	/*
-	 * Wait on our receive slot for some client to enqueue itself
-	 * in our send queue.
+	 * We loop to care for spurious wakeups, in case the
+	 * client times out before we unblock.
 	 */
-	info = xnsynch_sleep_on(&server->mrecv, timeout, XN_RELATIVE);
-	/*
-	 * XNRMID cannot happen, since well, the current task would be the
-	 * deleted object, so...
-	 */
-	if (info & XNTIMEO) {
-		err = -ETIMEDOUT;	/* Timeout. */
-		goto unlock_and_exit;
-	} else if (info & XNBREAK) {
-		err = -EINTR;	/* Unblocked. */
-		goto unlock_and_exit;
-	}
+	do {
+		/*
+		 * Wait on our receive slot for some client to enqueue
+		 * itself in our send queue.
+		 */
+		info = xnsynch_sleep_on(&server->mrecv, timeout, XN_RELATIVE);
+		/*
+		 * XNRMID cannot happen, since well, the current task
+		 * would be the deleted object, so...
+		 */
+		if (info & XNTIMEO) {
+			err = -ETIMEDOUT;	/* Timeout. */
+			goto unlock_and_exit;
+		}
+		if (info & XNBREAK) {
+			err = -EINTR;	/* Unblocked. */
+			goto unlock_and_exit;
+		}
+	} while (!xnsynch_pended_p(&server->msendq));
 
 	holder = getheadpq(xnsynch_wait_queue(&server->msendq));
 	/* There must be a valid holder since we waited for it. */
@@ -2034,7 +2091,9 @@ int rt_task_receive(RT_TASK_MCB *mcb_r, RTIME timeout)
  * field by the rt_task_send() service. If @a mcb_s is NULL, 0 will be
  * returned to the client into the status code field.
  *
- * @return O is returned upon success. Otherwise:
+ * @return Zero is returned upon success. Otherwise:
+ *
+ * - -EINVAL is returned if @a flowid is invalid.
  *
  * - -ENXIO is returned if @a flowid does not match the expected
  * identifier returned from the latest call of the current task to
@@ -2071,6 +2130,9 @@ int rt_task_reply(int flowid, RT_TASK_MCB *mcb_s)
 	if (!xnpod_primary_p())
 		return -EPERM;
 
+	if (flowid <= 0)
+		return -EINVAL;
+
 	server = xeno_current_task();
 
 	xnlock_get_irqsave(&nklock, s);
@@ -2091,7 +2153,7 @@ int rt_task_reply(int flowid, RT_TASK_MCB *mcb_s)
 			   client to be unblocked without transferring
 			   the ownership of the msendq object which
 			   must always belong to the server. */
-			xnpod_resume_thread(&client->thread_base, XNPEND);
+			xnpod_unblock_thread(&client->thread_base);
 			break;
 		}
 	}
@@ -2120,9 +2182,9 @@ int rt_task_reply(int flowid, RT_TASK_MCB *mcb_s)
 		   through the rescheduling is wanted. */
 		err = -ENOBUFS;
 
-	/* Copy back the actual size of the reply data, */
+	/* Fill in the reply block. */
+	client->wait_args.mps.mcb_r.flowid = flowid;
 	client->wait_args.mps.mcb_r.size = rsize;
-	/* And the status code. */
 	client->wait_args.mps.mcb_r.opcode = mcb_s ? mcb_s->opcode : 0;
 
 	/* That's it, we just need to start the rescheduling procedure
@@ -2465,28 +2527,28 @@ int rt_task_reply(int flowid, RT_TASK_MCB *mcb_s)
 
 /*@}*/
 
-EXPORT_SYMBOL(rt_task_create);
-EXPORT_SYMBOL(rt_task_start);
-EXPORT_SYMBOL(rt_task_suspend);
-EXPORT_SYMBOL(rt_task_resume);
-EXPORT_SYMBOL(rt_task_delete);
-EXPORT_SYMBOL(rt_task_yield);
-EXPORT_SYMBOL(rt_task_set_periodic);
-EXPORT_SYMBOL(rt_task_wait_period);
-EXPORT_SYMBOL(rt_task_set_priority);
-EXPORT_SYMBOL(rt_task_sleep);
-EXPORT_SYMBOL(rt_task_sleep_until);
-EXPORT_SYMBOL(rt_task_unblock);
-EXPORT_SYMBOL(rt_task_inquire);
-EXPORT_SYMBOL(rt_task_add_hook);
-EXPORT_SYMBOL(rt_task_remove_hook);
-EXPORT_SYMBOL(rt_task_catch);
-EXPORT_SYMBOL(rt_task_notify);
-EXPORT_SYMBOL(rt_task_set_mode);
-EXPORT_SYMBOL(rt_task_self);
-EXPORT_SYMBOL(rt_task_slice);
+EXPORT_SYMBOL_GPL(rt_task_create);
+EXPORT_SYMBOL_GPL(rt_task_start);
+EXPORT_SYMBOL_GPL(rt_task_suspend);
+EXPORT_SYMBOL_GPL(rt_task_resume);
+EXPORT_SYMBOL_GPL(rt_task_delete);
+EXPORT_SYMBOL_GPL(rt_task_yield);
+EXPORT_SYMBOL_GPL(rt_task_set_periodic);
+EXPORT_SYMBOL_GPL(rt_task_wait_period);
+EXPORT_SYMBOL_GPL(rt_task_set_priority);
+EXPORT_SYMBOL_GPL(rt_task_sleep);
+EXPORT_SYMBOL_GPL(rt_task_sleep_until);
+EXPORT_SYMBOL_GPL(rt_task_unblock);
+EXPORT_SYMBOL_GPL(rt_task_inquire);
+EXPORT_SYMBOL_GPL(rt_task_add_hook);
+EXPORT_SYMBOL_GPL(rt_task_remove_hook);
+EXPORT_SYMBOL_GPL(rt_task_catch);
+EXPORT_SYMBOL_GPL(rt_task_notify);
+EXPORT_SYMBOL_GPL(rt_task_set_mode);
+EXPORT_SYMBOL_GPL(rt_task_self);
+EXPORT_SYMBOL_GPL(rt_task_slice);
 #ifdef CONFIG_XENO_OPT_NATIVE_MPS
-EXPORT_SYMBOL(rt_task_send);
-EXPORT_SYMBOL(rt_task_receive);
-EXPORT_SYMBOL(rt_task_reply);
+EXPORT_SYMBOL_GPL(rt_task_send);
+EXPORT_SYMBOL_GPL(rt_task_receive);
+EXPORT_SYMBOL_GPL(rt_task_reply);
 #endif /* CONFIG_XENO_OPT_NATIVE_MPS */
