@@ -23,7 +23,7 @@
  * Thread scheduling services.
  *
  * Xenomai POSIX skin supports the scheduling policies SCHED_FIFO,
- * SCHED_RR, SCHED_SPORADIC and SCHED_OTHER.
+ * SCHED_RR, SCHED_SPORADIC, SCHED_TP and SCHED_OTHER.
  *
  * The SCHED_OTHER policy is mainly useful for user-space non-realtime
  * activities that need to synchronize with real-time activities.
@@ -38,6 +38,15 @@
  * The SCHED_SPORADIC policy provides a mean to schedule aperiodic or
  * sporadic threads in periodic-based systems.
  *
+ * The SCHED_TP policy divides the scheduling time into a recurring
+ * global frame, which is itself divided into an arbitrary number of
+ * time partitions. Only threads assigned to the current partition are
+ * deemed runnable, and scheduled according to a FIFO-based rule
+ * within this partition. When completed, the current partition is
+ * advanced automatically to the next one by the scheduler, and the
+ * global time frame recurs from the first partition defined, when the
+ * last partition has ended.
+ *
  * The scheduling policy and priority of a thread is set when creating a thread,
  * by using thread creation attributes (see pthread_attr_setinheritsched(),
  * pthread_attr_setschedpolicy() and pthread_attr_setschedparam()), or when the
@@ -57,8 +66,8 @@
  * This service returns the minimum priority of the scheduling policy @a
  * policy.
  *
- * @param policy scheduling policy, one of SCHED_FIFO, SCHED_RR, or
- * SCHED_OTHER.
+ * @param policy scheduling policy, one of SCHED_FIFO, SCHED_RR,
+ * SCHED_SPORADIC, SCHED_TP or SCHED_OTHER.
  *
  * @retval 0 on success;
  * @retval -1 with @a errno set if:
@@ -75,6 +84,7 @@ int sched_get_priority_min(int policy)
 	case SCHED_FIFO:
 	case SCHED_RR:
 	case SCHED_SPORADIC:
+	case SCHED_TP:
 		return PSE51_MIN_PRIORITY;
 
 	case SCHED_OTHER:
@@ -92,8 +102,8 @@ int sched_get_priority_min(int policy)
  * This service returns the maximum priority of the scheduling policy @a
  * policy.
  *
- * @param policy scheduling policy, one of SCHED_FIFO, SCHED_RR, or
- * SCHED_OTHER.
+ * @param policy scheduling policy, one of SCHED_FIFO, SCHED_RR,
+ * SCHED_SPORADIC, SCHED_TP or SCHED_OTHER.
  *
  * @retval 0 on success;
  * @retval -1 with @a errno set if:
@@ -110,6 +120,7 @@ int sched_get_priority_max(int policy)
 	case SCHED_FIFO:
 	case SCHED_RR:
 	case SCHED_SPORADIC:
+	case SCHED_TP:
 		return PSE51_MAX_PRIORITY;
 
 	case SCHED_OTHER:
@@ -215,8 +226,8 @@ int pthread_getschedparam(pthread_t tid, int *pol, struct sched_param *par)
  * that also supports Xenomai-specific or additional POSIX scheduling
  * policies, which are not available with the host Linux environment.
  *
- * Typically, SCHED_SPORADIC parameters can be retrieved from this
- * call.
+ * Typically, SCHED_SPORADIC or SCHED_TP parameters can be retrieved
+ * from this call.
  *
  * @param tid target thread;
  *
@@ -271,6 +282,13 @@ int pthread_getschedparam_ex(pthread_t tid, int *pol, struct sched_param_ex *par
 		goto unlock_and_exit;
 	}
 #endif
+#ifdef CONFIG_XENO_OPT_SCHED_TP
+	if (base_class == &xnsched_class_tp) {
+		*pol = SCHED_TP;
+		par->sched_tp_partition = thread->tps - thread->sched->tp.partitions;
+		goto unlock_and_exit;
+	}
+#endif
 
 unlock_and_exit:
 
@@ -295,8 +313,8 @@ unlock_and_exit:
  *
  * @param tid target thread;
  *
- * @param pol scheduling policy, one of SCHED_FIFO, SCHED_RR or
- * SCHED_OTHER;
+ * @param pol scheduling policy, one of SCHED_FIFO, SCHED_RR,
+ * SCHED_SPORADIC, SCHED_TP or SCHED_OTHER;
  *
  * @param par scheduling parameters address.
  *
@@ -356,6 +374,7 @@ int pthread_setschedparam(pthread_t tid, int pol, const struct sched_param *par)
 	case SCHED_OTHER:
 	case SCHED_FIFO:
 	case SCHED_SPORADIC:
+	case SCHED_TP:
 		xnpod_set_thread_tslice(&tid->threadbase, XN_INFINITE);
 		break;
 
@@ -390,8 +409,8 @@ int pthread_setschedparam(pthread_t tid, int pol, const struct sched_param *par)
  * that supports Xenomai-specific or additional POSIX scheduling
  * policies, which are not available with the host Linux environment.
  *
- * Typically, a Xenomai thread policy can be set to SCHED_SPORADIC
- * using this call.
+ * Typically, a Xenomai thread policy can be set to SCHED_SPORADIC or
+ * SCHED_TP using this call.
  *
  * @param tid target thread;
  *
@@ -457,11 +476,20 @@ int pthread_setschedparam_ex(pthread_t tid, int pol, const struct sched_param_ex
 		ret = -xnpod_set_thread_schedparam(&tid->threadbase,
 						   &xnsched_class_sporadic, &param);
 		break;
-#else
-		(void)param;
+#endif
+#ifdef CONFIG_XENO_OPT_SCHED_TP
+	case SCHED_TP:
+		xnpod_set_thread_tslice(&tid->threadbase, XN_INFINITE);
+		param.tp.prio = par->sched_priority;
+		param.tp.ptid = par->sched_tp_partition;
+		ret = -xnpod_set_thread_schedparam(&tid->threadbase,
+						   &xnsched_class_tp, &param);
+		break;
 #endif
 	}
 
+	(void)param;
+
 	xnpod_schedule();
 
 	xnlock_put_irqrestore(&nklock, s);
@@ -487,6 +515,140 @@ int sched_yield(void)
 	return 0;
 }
 
+#ifdef CONFIG_XENO_OPT_SCHED_TP
+
+static inline
+int set_tp_config(int cpu, union sched_config *config, size_t len)
+{
+	xnticks_t offset, duration, next_offset;
+	struct xnsched_tp_schedule *gps, *ogps;
+	struct xnsched_tp_window *w;
+	struct sched_tp_window *p;
+	struct xnsched *sched;
+	spl_t s;
+	int n;
+
+	gps = xnmalloc(sizeof(*gps) + config->tp.nr_windows * sizeof(*w));
+	if (gps == NULL)
+		goto fail;
+
+	for (n = 0, p = config->tp.windows, w = gps->pwins, next_offset = 0;
+	     n < config->tp.nr_windows; n++, p++, w++) {
+		/*
+		 * Time windows must be strictly contiguous. Holes may
+		 * be defined using windows assigned to the pseudo
+		 * partition #-1.
+		 */
+		offset = ts2ticks_ceil(&p->offset);
+		if (offset != next_offset)
+			goto cleanup_and_fail;
+
+		duration = ts2ticks_ceil(&p->duration);
+		if (duration <= 0)
+			goto cleanup_and_fail;
+
+		if (p->ptid < -1 ||
+		    p->ptid >= CONFIG_XENO_OPT_SCHED_TP_NRPART)
+			goto cleanup_and_fail;
+
+		w->w_offset = next_offset;
+		w->w_part = p->ptid;
+		next_offset += duration;
+	}
+
+	gps->pwin_nr = n;
+	gps->tf_duration = next_offset;
+	sched = xnpod_sched_slot(cpu);
+
+	xnlock_get_irqsave(&nklock, s);
+	ogps = xnsched_tp_set_schedule(sched, gps);
+	xnsched_tp_start_schedule(sched);
+	xnlock_put_irqrestore(&nklock, s);
+
+	if (ogps)
+		xnfree(ogps);
+
+	return 0;
+
+cleanup_and_fail:
+	xnfree(gps);
+fail:
+	return EINVAL;
+}
+
+#else /* !CONFIG_XENO_OPT_SCHED_TP */
+
+static inline
+int set_tp_config(int cpu, union sched_config *config, size_t len)
+{
+	return EINVAL;
+}
+
+#endif /* !CONFIG_XENO_OPT_SCHED_TP */
+
+/**
+ * Load CPU-specific scheduler settings for a given policy.
+ *
+ * Currently, this call only supports the SCHED_TP policy, for loading
+ * the temporal partitions. A configuration is strictly local to the
+ * target @a cpu, and may differ from other processors.
+ *
+ * @param cpu processor to load the configuration of.
+ *
+ * @param policy scheduling policy to which the configuration data
+ * applies. Currently, only SCHED_TP is valid.
+ *
+ * @param p a pointer to the configuration data to load for @a
+ * cpu, applicable to @a policy.
+ *
+ * Settings applicable to SCHED_TP:
+ *
+ * This call installs the temporal partitions for @a cpu.
+ *
+ * - config.tp.windows should be a non-null set of time windows,
+ * defining the scheduling time slots for @a cpu. Each window defines
+ * its offset from the start of the global time frame
+ * (windows[].offset), a duration (windows[].duration), and the
+ * partition id it applies to (windows[].ptid).
+ *
+ * Time windows must be strictly contiguous, i.e. windows[n].offset +
+ * windows[n].duration shall equal windows[n + 1].offset.
+ * If windows[].ptid is in the range
+ * [0..CONFIG_XENO_OPT_SCHED_TP_NRPART-1], SCHED_TP threads which
+ * belong to the partition being referred to may run for the duration
+ * of the time window.
+ *
+ * Time holes may be defined using windows assigned to the pseudo
+ * partition #-1, during which no SCHED_TP threads may be scheduled.
+ *
+ * - config.tp.nr_windows should define the number of elements present
+ * in the config.tp.windows[] array.
+ *
+ * @param len size of the configuration data (in bytes).
+ *
+ * @return 0 on success;
+ * @return an error number if:
+ * - EINVAL, @a cpu is invalid, @a policy is different from SCHED_TP,
+ * SCHED_TP support is not compiled in (see CONFIG_XENO_OPT_SCHED_TP),
+ * @a len is zero, or @a p contains invalid parameters.
+ * - ENOMEM, lack of memory to perform the operation.
+ */
+int sched_setconfig_np(int cpu, int policy,
+		       union sched_config *config, size_t len)
+{
+	int ret;
+
+	switch (policy)	{
+	case SCHED_TP:
+		ret = set_tp_config(cpu, config, len);
+		break;
+	default:
+		ret = EINVAL;
+	}
+
+	return ret;
+}
+
 /*@}*/
 
 EXPORT_SYMBOL_GPL(sched_get_priority_min);
@@ -497,3 +659,4 @@ EXPORT_SYMBOL_GPL(pthread_getschedparam_ex);
 EXPORT_SYMBOL_GPL(pthread_setschedparam);
 EXPORT_SYMBOL_GPL(pthread_setschedparam_ex);
 EXPORT_SYMBOL_GPL(sched_yield);
+EXPORT_SYMBOL_GPL(sched_setconfig_np);
