@@ -402,9 +402,9 @@ static int __pthread_wait_np(struct pt_regs *regs)
 	err = -pthread_wait_np(&overruns);
 
 	if (__xn_reg_arg1(regs) && (err == 0 || err == -ETIMEDOUT))
-		if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg1(regs),
-					   &overruns, sizeof(overruns)))
-			err = -EFAULT;
+		__xn_put_user(overruns,
+			      (unsigned long __user *)__xn_reg_arg1(regs));
+
 	return err;
 }
 
@@ -422,6 +422,7 @@ static int __pthread_set_name_np(struct pt_regs *regs)
 {
 	char name[XNOBJECT_NAME_LEN];
 	struct pse51_hkey hkey;
+	struct task_struct *p;
 	pthread_t k_tid;
 
 	if (__xn_safe_strncpy_from_user(name,
@@ -434,6 +435,11 @@ static int __pthread_set_name_np(struct pt_regs *regs)
 	hkey.u_tid = __xn_reg_arg1(regs);
 	hkey.mm = current->mm;
 	k_tid = __pthread_find(&hkey);
+	if (k_tid) {
+		p = xnthread_user_task(&k_tid->threadbase);
+		strncpy(p->comm, name, sizeof(p->comm));
+		p->comm[sizeof(p->comm) - 1] = '\0';
+	}
 
 	return -pthread_set_name_np(k_tid, name);
 }
@@ -603,11 +609,16 @@ static int __sem_open(struct pt_regs *regs)
 	union __xeno_sem *sm;
 	pse51_assoc_t *assoc;
 	unsigned long uaddr;
+	pse51_queues_t *q;
 	pse51_usem_t *usm;
 	int oflags, err;
 	long len;
 	spl_t s;
 
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
+
 	if (__xn_safe_copy_from_user(&uaddr,
 				     (void __user *)__xn_reg_arg1(regs), sizeof(uaddr)))
 		return -EFAULT;
@@ -637,8 +648,7 @@ static int __sem_open(struct pt_regs *regs)
 		return -thread_get_errno();
 
 	xnlock_get_irqsave(&pse51_assoc_lock, s);
-	assoc = pse51_assoc_lookup(&pse51_queues()->usems,
-				   (u_long)sm->shadow_sem.sem);
+	assoc = pse51_assoc_lookup(&q->usems, (u_long)sm->shadow_sem.sem);
 
 	if (assoc) {
 		usm = assoc2usem(assoc);
@@ -660,10 +670,7 @@ static int __sem_open(struct pt_regs *regs)
 	usm->refcnt = 1;
 
 	xnlock_get_irqsave(&pse51_assoc_lock, s);
-	assoc =
-	    pse51_assoc_lookup(&pse51_queues()->usems,
-			       (u_long)sm->shadow_sem.sem);
-
+	assoc = pse51_assoc_lookup(&q->usems, (u_long)sm->shadow_sem.sem);
 	if (assoc) {
 		assoc2usem(assoc)->refcnt++;
 		xnlock_put_irqrestore(&nklock, s);
@@ -672,8 +679,7 @@ static int __sem_open(struct pt_regs *regs)
 		goto got_usm;
 	}
 
-	pse51_assoc_insert(&pse51_queues()->usems,
-			   &usm->assoc, (u_long)sm->shadow_sem.sem);
+	pse51_assoc_insert(&q->usems, &usm->assoc, (u_long)sm->shadow_sem.sem);
 	xnlock_put_irqrestore(&pse51_assoc_lock, s);
 
       got_usm:
@@ -696,9 +702,14 @@ static int __sem_close(struct pt_regs *regs)
 	union __xeno_sem sm;
 	unsigned long uaddr;
 	int closed = 0, err;
+	pse51_queues_t *q;
 	pse51_usem_t *usm;
 	spl_t s;
 
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
+
 	uaddr = (unsigned long)__xn_reg_arg1(regs);
 
 	if (__xn_safe_copy_from_user(&sm.shadow_sem,
@@ -707,10 +718,7 @@ static int __sem_close(struct pt_regs *regs)
 
 	xnlock_get_irqsave(&pse51_assoc_lock, s);
 
-	assoc =
-	    pse51_assoc_lookup(&pse51_queues()->usems,
-			       (u_long)sm.shadow_sem.sem);
-
+	assoc = pse51_assoc_lookup(&q->usems, (u_long)sm.shadow_sem.sem);
 	if (!assoc) {
 		xnlock_put_irqrestore(&pse51_assoc_lock, s);
 		return -EINVAL;
@@ -721,8 +729,7 @@ static int __sem_close(struct pt_regs *regs)
 	err = sem_close(&sm.native_sem);
 
 	if (!err && (closed = (--usm->refcnt == 0)))
-		pse51_assoc_remove(&pse51_queues()->usems,
-				   (u_long)sm.shadow_sem.sem);
+		pse51_assoc_remove(&q->usems, (u_long)sm.shadow_sem.sem);
 
 	xnlock_put_irqrestore(&pse51_assoc_lock, s);
 
@@ -1692,11 +1699,16 @@ static int __mq_open(struct pt_regs *regs)
 	struct mq_attr locattr, *attr;
 	char name[PSE51_MAXNAME];
 	pse51_ufd_t *assoc;
+	pse51_queues_t *q;
 	int err, oflags;
 	mqd_t kqd, uqd;
 	unsigned len;
 	mode_t mode;
 
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
+
 	len = __xn_safe_strncpy_from_user(name,
 					  (const char __user *)__xn_reg_arg1(regs),
 					  sizeof(name));
@@ -1736,9 +1748,7 @@ static int __mq_open(struct pt_regs *regs)
 
 	assoc->kfd = kqd;
 
-	err = pse51_assoc_insert(&pse51_queues()->uqds,
-				 &assoc->assoc, (u_long)uqd);
-
+	err = pse51_assoc_insert(&q->uqds, &assoc->assoc, (u_long)uqd);
 	if (err) {
 		xnfree(assoc);
 		mq_close(kqd);
@@ -1750,13 +1760,17 @@ static int __mq_open(struct pt_regs *regs)
 static int __mq_close(struct pt_regs *regs)
 {
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	mqd_t uqd;
 	int err;
 
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
+
 	uqd = (mqd_t) __xn_reg_arg1(regs);
 
-	assoc = pse51_assoc_remove(&pse51_queues()->uqds, (u_long)uqd);
-
+	assoc = pse51_assoc_remove(&q->uqds, (u_long)uqd);
 	if (!assoc)
 		return -EBADF;
 
@@ -1790,11 +1804,15 @@ static int __mq_getattr(struct pt_regs *regs)
 {
 	pse51_assoc_t *assoc;
 	struct mq_attr attr;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 	int err;
 
-	assoc = pse51_assoc_lookup(&pse51_queues()->uqds,
-				   (u_long)__xn_reg_arg1(regs));
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
+
+	assoc = pse51_assoc_lookup(&q->uqds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -1813,11 +1831,15 @@ static int __mq_setattr(struct pt_regs *regs)
 {
 	struct mq_attr attr, oattr;
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 	int err;
 
-	assoc = pse51_assoc_lookup(&pse51_queues()->uqds,
-				   (u_long)__xn_reg_arg1(regs));
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
+
+	assoc = pse51_assoc_lookup(&q->uqds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -1842,19 +1864,21 @@ static int __mq_setattr(struct pt_regs *regs)
 static int __mq_send(struct pt_regs *regs)
 {
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	pse51_msg_t *msg;
 	pse51_ufd_t *ufd;
 	pse51_mq_t *mq;
 	unsigned prio;
 	size_t len;
 
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
+
 	len = (size_t) __xn_reg_arg3(regs);
 	prio = __xn_reg_arg4(regs);
 
-	assoc =
-	    pse51_assoc_lookup(&pse51_queues()->uqds,
-			       (u_long)__xn_reg_arg1(regs));
-
+	assoc = pse51_assoc_lookup(&q->uqds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -1883,19 +1907,21 @@ static int __mq_timedsend(struct pt_regs *regs)
 {
 	struct timespec timeout, *timeoutp;
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	pse51_msg_t *msg;
 	pse51_ufd_t *ufd;
 	pse51_mq_t *mq;
 	unsigned prio;
 	size_t len;
 
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
+
 	len = (size_t) __xn_reg_arg3(regs);
 	prio = __xn_reg_arg4(regs);
 
-	assoc =
-	    pse51_assoc_lookup(&pse51_queues()->uqds,
-			       (u_long)__xn_reg_arg1(regs));
-
+	assoc = pse51_assoc_lookup(&q->uqds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -1931,6 +1957,7 @@ static int __mq_timedsend(struct pt_regs *regs)
 static int __mq_receive(struct pt_regs *regs)
 {
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 	pse51_msg_t *msg;
 	pse51_mq_t *mq;
@@ -1938,8 +1965,11 @@ static int __mq_receive(struct pt_regs *regs)
 	ssize_t len;
 	int err;
 
-	assoc = pse51_assoc_lookup(&pse51_queues()->uqds,
-				   (u_long)__xn_reg_arg1(regs));
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
+
+	assoc = pse51_assoc_lookup(&q->uqds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -1988,6 +2018,7 @@ static int __mq_timedreceive(struct pt_regs *regs)
 {
 	struct timespec timeout, *timeoutp;
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 	pse51_msg_t *msg;
 	pse51_mq_t *mq;
@@ -1995,8 +2026,11 @@ static int __mq_timedreceive(struct pt_regs *regs)
 	ssize_t len;
 	int err;
 
-	assoc = pse51_assoc_lookup(&pse51_queues()->uqds,
-				   (u_long)__xn_reg_arg1(regs));
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
+
+	assoc = pse51_assoc_lookup(&q->uqds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -2050,12 +2084,14 @@ static int __mq_notify(struct pt_regs *regs)
 {
 	pse51_assoc_t *assoc;
 	struct sigevent sev;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 
-	assoc =
-	    pse51_assoc_lookup(&pse51_queues()->uqds,
-			       (u_long)__xn_reg_arg1(regs));
+	q = pse51_queues();
+	if(!q)
+		return -EPERM;
 
+	assoc = pse51_assoc_lookup(&q->uqds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -2290,9 +2326,9 @@ static int __timer_getoverrun(struct pt_regs *regs)
 #ifdef CONFIG_XENO_OPT_POSIX_SELECT
 static int fd_valid_p(int fd)
 {
-	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 #if defined(CONFIG_XENO_SKIN_RTDM) || defined (CONFIG_XENO_SKIN_RTDM_MODULE)
-	const int rtdm_fd_start = FD_SETSIZE - RTDM_FD_MAX;
+	const int rtdm_fd_start = __FD_SETSIZE - RTDM_FD_MAX;
 
 	if (fd >= rtdm_fd_start) {
 		struct rtdm_dev_context *ctx;
@@ -2305,8 +2341,11 @@ static int fd_valid_p(int fd)
 	}
 #endif /* RTDM */
 
-	assoc = pse51_assoc_lookup(&pse51_queues()->uqds, fd);
-	return assoc != NULL;
+	q = pse51_queues();
+	if (!q)
+		return 0;
+
+	return pse51_assoc_lookup(&q->uqds, fd) != NULL;
 }
 
 static int first_fd_valid_p(fd_set *fds[XNSELECT_MAX_TYPES], int nfds)
@@ -2326,15 +2365,20 @@ static int first_fd_valid_p(fd_set *fds[XNSELECT_MAX_TYPES], int nfds)
 static int select_bind_one(struct xnselector *selector, unsigned type, int fd)
 {
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 #if defined(CONFIG_XENO_SKIN_RTDM) || defined (CONFIG_XENO_SKIN_RTDM_MODULE)
-	const int rtdm_fd_start = FD_SETSIZE - RTDM_FD_MAX;
+	const int rtdm_fd_start = __FD_SETSIZE - RTDM_FD_MAX;
 
 	if (fd >= rtdm_fd_start)
 		return rtdm_select_bind(fd - rtdm_fd_start,
 					selector, type, fd);
 #endif /* CONFIG_XENO_SKIN_RTDM */
 
-	assoc = pse51_assoc_lookup(&pse51_queues()->uqds, fd);
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
+
+	assoc = pse51_assoc_lookup(&q->uqds, fd);
 	if (!assoc)
 		return -EBADF;
 
@@ -2401,7 +2445,7 @@ static int __select(struct pt_regs *regs)
 	}
 
 	nfds = __xn_reg_arg1(regs);
-	fds_size = __FDELT(nfds + __NFDBITS - 1) * sizeof(long);
+	fds_size = __FDELT__(nfds + __NFDBITS__ - 1) * sizeof(long);
 
 	for (i = 0; i < XNSELECT_MAX_TYPES; i++)
 		if (ufd_sets[i]) {
@@ -2457,7 +2501,7 @@ static int __select(struct pt_regs *regs)
 			return -EFAULT;
 	}
 
-	if (err > 0)
+	if (err >= 0)
 		for (i = 0; i < XNSELECT_MAX_TYPES; i++)
 			if (ufd_sets[i]
 			    && __xn_copy_to_user((void __user *) ufd_sets[i],
@@ -2477,9 +2521,14 @@ static int __shm_open(struct pt_regs *regs)
 	char name[PSE51_MAXNAME];
 	int ufd, kfd, oflag, err;
 	pse51_ufd_t *assoc;
+	pse51_queues_t *q;
 	unsigned len;
 	mode_t mode;
 
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
+
 	len = __xn_safe_strncpy_from_user(name,
 					  (const char __user *)__xn_reg_arg1(regs),
 					  sizeof(name));
@@ -2510,10 +2559,7 @@ static int __shm_open(struct pt_regs *regs)
 
 	ufd = (int)__xn_reg_arg4(regs);
 
-	err =
-	    pse51_assoc_insert(&pse51_queues()->ufds, &assoc->assoc,
-			       (u_long)ufd);
-
+	err = pse51_assoc_insert(&q->ufds, &assoc->assoc, (u_long)ufd);
 	if (err) {
 		xnfree(assoc);
 		close(kfd);
@@ -2547,13 +2593,15 @@ static int __shm_unlink(struct pt_regs *regs)
 static int __shm_close(struct pt_regs *regs)
 {
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 	int err;
 
-	assoc =
-	    pse51_assoc_remove(&pse51_queues()->ufds,
-			       (u_long)__xn_reg_arg1(regs));
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
 
+	assoc = pse51_assoc_remove(&q->ufds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -2569,16 +2617,18 @@ static int __shm_close(struct pt_regs *regs)
 static int __ftruncate(struct pt_regs *regs)
 {
 	pse51_assoc_t *assoc;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 	off_t len;
 	int err;
 
-	len = (off_t) __xn_reg_arg2(regs);
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
 
-	assoc =
-	    pse51_assoc_lookup(&pse51_queues()->ufds,
-			       (u_long)__xn_reg_arg1(regs));
+	len = (off_t)__xn_reg_arg2(regs);
 
+	assoc = pse51_assoc_lookup(&q->ufds, (u_long)__xn_reg_arg1(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -2603,18 +2653,20 @@ static int __mmap_prologue(struct pt_regs *regs)
 	pse51_mmap_param_t mmap_param;
 	pse51_assoc_t *assoc;
 	struct xnheap *heap;
+	pse51_queues_t *q;
 	pse51_ufd_t *ufd;
 	size_t len;
 	off_t off;
 	int err;
 
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
+
 	len = (size_t) __xn_reg_arg1(regs);
 	off = (off_t) __xn_reg_arg3(regs);
 
-	assoc =
-	    pse51_assoc_lookup(&pse51_queues()->ufds,
-			       (u_long)__xn_reg_arg2(regs));
-
+	assoc = pse51_assoc_lookup(&q->ufds, (u_long)__xn_reg_arg2(regs));
 	if (!assoc)
 		return -EBADF;
 
@@ -2696,14 +2748,19 @@ static int __munmap_prologue(struct pt_regs *regs)
 	pse51_assoc_t *assoc;
 	unsigned long uaddr;
 	pse51_umap_t *umap;
+	pse51_queues_t *q;
 	xnheap_t *heap;
 	size_t len;
 	int err;
 
+	q = pse51_queues();
+	if (!q)
+		return -EPERM;
+
 	uaddr = (unsigned long)__xn_reg_arg1(regs);
 	len = (size_t) __xn_reg_arg2(regs);
 
-	assoc = pse51_assoc_lookup(&pse51_queues()->umaps, uaddr);
+	assoc = pse51_assoc_lookup(&q->umaps, uaddr);
 
 	if (!assoc)
 		return -EBADF;
@@ -2772,6 +2829,47 @@ static int __munmap_epilogue(struct pt_regs *regs)
 
 #endif /* !CONFIG_XENO_OPT_POSIX_SHM */
 
+#ifdef CONFIG_XENO_OPT_SCHED_TP
+/*
+ * int __sched_setconfig_np(int cpu, int policy, union sched_config *p, size_t len)
+ */
+static int __sched_setconfig_np(struct pt_regs *regs)
+{
+	union sched_config __user *u_config, *buf;
+	int len, ret, cpu, policy;
+
+	cpu = __xn_reg_arg1(regs);
+	if (cpu < 0 || cpu >= NR_CPUS || !cpu_online(cpu))
+		return -EINVAL;
+
+	policy = __xn_reg_arg2(regs);
+	u_config = (union sched_config __user *)__xn_reg_arg3(regs);
+	len = __xn_reg_arg4(regs);
+	if (len == 0)
+		return -EINVAL;
+
+	buf = xnmalloc(len);
+	if (buf == NULL)
+		return -ENOMEM;
+
+	if (__xn_safe_copy_from_user(buf, (void __user *)u_config, len)) {
+		ret = -EFAULT;
+		goto out;
+	}
+
+	ret = -sched_setconfig_np(cpu, policy, buf, len);
+out:
+	xnfree(buf);
+
+	return ret;
+}
+
+#else /* !CONFIG_XENO_OPT_SCHED_TP */
+
+#define __sched_setconfig_np        __pse51_call_not_available
+
+#endif /* !CONFIG_XENO_OPT_SCHED_TP */
+
 int __pse51_call_not_available(struct pt_regs *regs)
 {
 	return -ENOSYS;
@@ -2883,6 +2981,7 @@ static xnsysent_t __systab[] = {
 	[__pse51_condattr_setpshared] =
 	    {&__pthread_condattr_setpshared, __xn_exec_any},
 	[__pse51_select] = {&__select, __xn_exec_primary},
+	[__pse51_sched_setconfig_np] = {&__sched_setconfig_np, __xn_exec_any},
 };
 
 static void __shadow_delete_hook(xnthread_t *thread)
