@@ -2,7 +2,7 @@
  * @file
  * This file is part of the Xenomai project.
  *
- * @note Copyright (C) 2007 Philippe Gerum <rpm@xenomai.org> 
+ * @note Copyright (C) 2007 Philippe Gerum <rpm@xenomai.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@ static int __ui_cre_tsk(struct pt_regs *regs)
 {
 	xncompletion_t __user *u_completion;
 	struct task_struct *p = current;
-	unsigned long __user *u_mode;
+	unsigned long __user *u_mode_offset;
 	uitask_t *task;
 	T_CTSK pk_ctsk;
 	ID tskid;
@@ -56,7 +56,7 @@ static int __ui_cre_tsk(struct pt_regs *regs)
 	/* Completion descriptor our parent thread is pending on. */
 	u_completion = (xncompletion_t __user *)__xn_reg_arg3(regs);
 
-	u_mode = (unsigned long __user *)__xn_reg_arg4(regs);
+	u_mode_offset = (unsigned long __user *)__xn_reg_arg4(regs);
 
 	err = cre_tsk(tskid, &pk_ctsk);
 
@@ -78,7 +78,8 @@ static int __ui_cre_tsk(struct pt_regs *regs)
 		 * recycled before we could map it; however, the risk
 		 * is mitigated by consistency checks performed in
 		 * xnshadow_map(). */
-		return xnshadow_map(&task->threadbase, u_completion, u_mode);
+		return xnshadow_map(&task->threadbase,
+				    u_completion, u_mode_offset);
 	}
 
       fail:
@@ -815,7 +816,6 @@ static struct xnskin_props __props = {
 	.nrcalls = sizeof(__systab) / sizeof(__systab[0]),
 	.systab = __systab,
 	.eventcb = &ui_shadow_eventcb,
-	.sig_unqueue = NULL,
 	.timebasep = &ui_tbase,
 	.module = THIS_MODULE
 };
