@@ -854,23 +854,23 @@ unlock_and_exit:
 	return err;
 }
 
-EXPORT_SYMBOL(cre_tsk);
-EXPORT_SYMBOL(del_tsk);
-EXPORT_SYMBOL(sta_tsk);
-EXPORT_SYMBOL(ext_tsk);
-EXPORT_SYMBOL(exd_tsk);
-EXPORT_SYMBOL(ter_tsk);
-EXPORT_SYMBOL(dis_dsp);
-EXPORT_SYMBOL(ena_dsp);
-EXPORT_SYMBOL(chg_pri);
-EXPORT_SYMBOL(rot_rdq);
-EXPORT_SYMBOL(rel_wai);
-EXPORT_SYMBOL(get_tid);
-EXPORT_SYMBOL(ref_tsk);
-EXPORT_SYMBOL(sus_tsk);
-EXPORT_SYMBOL(rsm_tsk);
-EXPORT_SYMBOL(frsm_tsk);
-EXPORT_SYMBOL(slp_tsk);
-EXPORT_SYMBOL(tslp_tsk);
-EXPORT_SYMBOL(wup_tsk);
-EXPORT_SYMBOL(can_wup);
+EXPORT_SYMBOL_GPL(cre_tsk);
+EXPORT_SYMBOL_GPL(del_tsk);
+EXPORT_SYMBOL_GPL(sta_tsk);
+EXPORT_SYMBOL_GPL(ext_tsk);
+EXPORT_SYMBOL_GPL(exd_tsk);
+EXPORT_SYMBOL_GPL(ter_tsk);
+EXPORT_SYMBOL_GPL(dis_dsp);
+EXPORT_SYMBOL_GPL(ena_dsp);
+EXPORT_SYMBOL_GPL(chg_pri);
+EXPORT_SYMBOL_GPL(rot_rdq);
+EXPORT_SYMBOL_GPL(rel_wai);
+EXPORT_SYMBOL_GPL(get_tid);
+EXPORT_SYMBOL_GPL(ref_tsk);
+EXPORT_SYMBOL_GPL(sus_tsk);
+EXPORT_SYMBOL_GPL(rsm_tsk);
+EXPORT_SYMBOL_GPL(frsm_tsk);
+EXPORT_SYMBOL_GPL(slp_tsk);
+EXPORT_SYMBOL_GPL(tslp_tsk);
+EXPORT_SYMBOL_GPL(wup_tsk);
+EXPORT_SYMBOL_GPL(can_wup);
