@@ -2,7 +2,7 @@
  * @file
  * This file is part of the Xenomai project.
  *
- * @note Copyright (C) 2006 Philippe Gerum <rpm@xenomai.org> 
+ * @note Copyright (C) 2006 Philippe Gerum <rpm@xenomai.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,7 +47,7 @@ static int __muxid;
  * a1: int tid;
  * a2: int prio;
  * a3: int mode;
- * a4: unsigned long *shadow_mode;
+ * a4: unsigned long *shadow_mode_offset;
  * }
  */
 
@@ -867,7 +867,7 @@ static int __sc_hcreate(struct pt_regs *regs)
 		hdesc.hid = hid;
 		hdesc.hcb = &heap->sysheap;
 		hdesc.hsize = xnheap_extentsize(&heap->sysheap);
-		xnheap_area_set(&hdesc, xnheap_base_memory(&heap->sysheap));
+		hdesc.area = xnheap_base_memory(&heap->sysheap);
 		xnlock_put_irqrestore(&nklock, s);
 
 		if (__xn_safe_copy_to_user((void __user *)__xn_reg_arg3(regs),
@@ -1076,7 +1076,7 @@ static int __sc_pcreate(struct pt_regs *regs)
 		pdesc.pid = pid;
 		pdesc.ptcb = ptheap;
 		pdesc.ptsize = xnheap_extentsize(ptheap);
-		xnheap_area_set(&pdesc, xnheap_base_memory(ptheap));
+		pdesc.area = xnheap_base_memory(ptheap);
 		xnlock_put_irqrestore(&nklock, s);
 
 		return __xn_safe_copy_to_user((void __user *)__xn_reg_arg4(regs),
@@ -1310,7 +1310,6 @@ static struct xnskin_props __props = {
 	.nrcalls = sizeof(__systab) / sizeof(__systab[0]),
 	.systab = __systab,
 	.eventcb = NULL,
-	.sig_unqueue = NULL,
 	.timebasep = &vrtx_tbase,
 	.module = THIS_MODULE
 };
