@@ -26,6 +26,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <syslog.h>
+#include <signal.h>
 
 #include <rtdk.h>
 #include <nucleus/types.h>	/* For BITS_PER_LONG */
@@ -47,13 +48,14 @@
 #define RT_PRINT_SYSLOG_STREAM		NULL
 
 #define RT_PRINT_MODE_FORMAT		0
-#define RT_PRINT_MODE_PUTS		1
+#define RT_PRINT_MODE_FWRITE		1
 
 struct entry_head {
 	FILE *dest;
 	uint32_t seq_no;
 	int priority;
-	char text[1];
+	size_t len;
+	char data[0];
 } __attribute__((packed));
 
 struct print_buffer {
@@ -90,13 +92,16 @@ static unsigned pool_buf_size;
 static unsigned long pool_start, pool_len;
 #endif /* CONFIG_XENO_FASTSYNCH */
 
+static pthread_once_t init_once = PTHREAD_ONCE_INIT;
+
 static void cleanup_buffer(struct print_buffer *buffer);
 static void print_buffers(void);
+static void spawn_printer_thread(void);
 
 /* *** rt_print API *** */
 
 static int vprint_to_buffer(FILE *stream, int priority, unsigned int mode,
-			    const char *format, va_list args)
+			    size_t sz, const char *format, va_list args)
 {
 	struct print_buffer *buffer = pthread_getspecific(buffer_key);
 	off_t write_pos, read_pos;
@@ -136,7 +141,7 @@ static int vprint_to_buffer(FILE *stream, int priority, unsigned int mode,
 			head = buffer->ring + write_pos;
 			head->seq_no = seq_no;
 			head->priority = 0;
-			head->text[0] = 0;
+			head->len = 0;
 
 			/* Forward to the ring buffer start */
 			write_pos = 0;
@@ -156,24 +161,35 @@ static int vprint_to_buffer(FILE *stream, int priority, unsigned int mode,
 	head = buffer->ring + write_pos;
 
 	if (mode == RT_PRINT_MODE_FORMAT) {
-		res = vsnprintf(head->text, len, format, args);
+		if (stream != RT_PRINT_SYSLOG_STREAM) {
+			/* We do not need the terminating \0 */
+			res = vsnprintf(head->data, len, format, args);
 
-		if (res < len) {
-			/* Text was written completely, res contains its
-			   length */
-			len = res;
+			if (res < len) {
+				/* Text was written completely, res contains its
+				   length */
+				len = res;
+			} else {
+				/* Text was truncated */
+				res = len;
+			}
 		} else {
-			/* Text was truncated, remove closing \0 that
-			   entry_head already includes */
-			len--;
-			res = len;
+			/* We DO need the terminating \0 */
+			res = vsnprintf(head->data, len, format, args);
+
+			if (res < len) {
+				/* Text was written completely, res contains its
+				   length */
+				len = res + 1;
+			} else {
+				/* Text was truncated */
+				res = len;
+			}
 		}
-	} else if (len >= 2) {
-		str_len = strlen(format);
-		len = (str_len < len - 2) ? str_len : len - 2;
-		strncpy(head->text, format, len);
-		head->text[len++] = '\n';
-		head->text[len] = 0;
+	} else if (len >= 1) {
+		str_len = sz;
+		len = (str_len < len) ? str_len : len;
+		memcpy(head->data, format, len);
 	} else
 		len = 0;
 
@@ -182,6 +198,7 @@ static int vprint_to_buffer(FILE *stream, int priority, unsigned int mode,
 		head->seq_no = ++seq_no;
 		head->priority = priority;
 		head->dest = stream;
+		head->len = len;
 
 		/* Move forward by text and head length */
 		write_pos += len + sizeof(struct entry_head);
@@ -194,7 +211,7 @@ static int vprint_to_buffer(FILE *stream, int priority, unsigned int mode,
 		head = buffer->ring + write_pos;
 		head->seq_no = seq_no;
 		head->priority = priority;
-		head->text[0] = 0;
+		head->len = 0;
 
 		write_pos = 0;
 	}
@@ -208,13 +225,13 @@ static int vprint_to_buffer(FILE *stream, int priority, unsigned int mode,
 }
 
 static int print_to_buffer(FILE *stream, int priority, unsigned int mode,
-			   const char *format, ...)
+			   size_t sz, const char *format, ...)
 {
 	va_list args;
 	int ret;
 
 	va_start(args, format);
-	ret = vprint_to_buffer(stream, priority, mode, format, args);
+	ret = vprint_to_buffer(stream, priority, mode, sz, format, args);
 	va_end(args);
 
 	return ret;
@@ -222,7 +239,8 @@ static int print_to_buffer(FILE *stream, int priority, unsigned int mode,
 
 int rt_vfprintf(FILE *stream, const char *format, va_list args)
 {
-	return vprint_to_buffer(stream, 0, RT_PRINT_MODE_FORMAT, format, args);
+	return vprint_to_buffer(stream, 0,
+				RT_PRINT_MODE_FORMAT, 0, format, args);
 }
 
 int rt_vprintf(const char *format, va_list args)
@@ -254,25 +272,60 @@ int rt_printf(const char *format, ...)
 	return n;
 }
 
+int rt_fputs(const char *s, FILE *stream)
+{
+	return print_to_buffer(stream, 0, RT_PRINT_MODE_FWRITE, strlen(s), s);
+}
+
 int rt_puts(const char *s)
 {
-	return print_to_buffer(stdout, 0, RT_PRINT_MODE_PUTS, s);
+	int res;
+
+	res = rt_fputs(s, stdout);
+	if (res < 0)
+		return res;
+
+	return print_to_buffer(stdout, 0, RT_PRINT_MODE_FWRITE, 1, "\n");
 }
 
+int rt_fputc(int c, FILE *stream)
+{
+	unsigned char uc = c;
+	int rc;
+
+	rc = print_to_buffer(stream, 0, RT_PRINT_MODE_FWRITE, 1, (char *)&uc);
+	if (rc < 0)
+		return EOF;
+
+	return (int)uc;
+}
+
+int rt_putchar(int c)
+{
+	return rt_fputc(c, stdout);
+}
+
+size_t rt_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream)
+{
+	print_to_buffer(stream, 0, RT_PRINT_MODE_FWRITE, size * nmemb, ptr);
+	return nmemb;
+}
+
+
 void rt_syslog(int priority, const char *format, ...)
 {
 	va_list args;
 
 	va_start(args, format);
 	vprint_to_buffer(RT_PRINT_SYSLOG_STREAM, priority,
-			 RT_PRINT_MODE_FORMAT, format, args);
+			 RT_PRINT_MODE_FORMAT, 0, format, args);
 	va_end(args);
 }
 
 void rt_vsyslog(int priority, const char *format, va_list args)
 {
 	vprint_to_buffer(RT_PRINT_SYSLOG_STREAM, priority,
-			 RT_PRINT_MODE_FORMAT, format, args);
+			 RT_PRINT_MODE_FORMAT, 0, format, args);
 }
 
 static void set_buffer_name(struct print_buffer *buffer, const char *name)
@@ -318,6 +371,8 @@ int rt_print_init(size_t buffer_size, const char *buffer_name)
 	unsigned long old_bitmap;
 	unsigned j;
 
+	pthread_once(&init_once, spawn_printer_thread);
+
 	if (!size)
 		size = default_buffer_size;
 	else if (size < RT_PRINT_LINE_BREAK)
@@ -389,6 +444,8 @@ int rt_print_init(size_t buffer_size, const char *buffer_name)
 void rt_print_auto_init(int enable)
 {
 	auto_init = enable;
+	if (enable)
+		pthread_once(&init_once, spawn_printer_thread);
 }
 
 void rt_print_cleanup(void)
@@ -406,6 +463,7 @@ void rt_print_cleanup(void)
 	}
 
 	pthread_cancel(printer_thread);
+	printer_thread = 0;
 }
 
 const char *rt_print_buffer_name(void)
@@ -532,7 +590,7 @@ static void print_buffers(void)
 	struct print_buffer *buffer;
 	struct entry_head *head;
 	off_t read_pos;
-	int len;
+	int len, ret;
 
 	while (1) {
 		buffer = get_next_buffer();
@@ -541,16 +599,18 @@ static void print_buffers(void)
 
 		read_pos = buffer->read_pos;
 		head = buffer->ring + read_pos;
-		len = strlen(head->text);
+		len = head->len;
 
 		if (len) {
 			/* Print out non-empty entry and proceed */
 			/* Check if output goes to syslog */
 			if (head->dest == RT_PRINT_SYSLOG_STREAM) {
-				syslog(head->priority, "%s", head->text);
+				syslog(head->priority,
+				       "%s", head->data);
 			} else {
-				/* Output goes to specified stream */
-				fprintf(head->dest, "%s", head->text);
+				ret = fwrite(head->data,
+					     head->len, 1, head->dest);
+				(void)ret;
 			}
 
 			read_pos += sizeof(*head) + len;
@@ -569,9 +629,21 @@ static void print_buffers(void)
 	}
 }
 
+static void unlock(void *cookie)
+{
+	pthread_mutex_t *mutex = (pthread_mutex_t *)cookie;
+	pthread_mutex_unlock(mutex);
+}
+
 static void *printer_loop(void *arg)
 {
+	sigset_t mask;
+
+	sigfillset(&mask);
+	pthread_sigmask(SIG_BLOCK, &mask, NULL);
+
 	while (1) {
+		pthread_cleanup_push(unlock, &buffer_lock);
 		pthread_mutex_lock(&buffer_lock);
 
 		while (buffers == 0)
@@ -579,7 +651,7 @@ static void *printer_loop(void *arg)
 
 		print_buffers();
 
-		pthread_mutex_unlock(&buffer_lock);
+		pthread_cleanup_pop(1);
 
 		nanosleep(&print_period, NULL);
 	}
@@ -593,6 +665,7 @@ static void spawn_printer_thread(void)
 
 	pthread_attr_init(&thattr);
 	pthread_attr_setstacksize(&thattr, xeno_stacksize(0));
+	pthread_attr_setdetachstate(&thattr, PTHREAD_CREATE_DETACHED);
 	pthread_create(&printer_thread, &thattr, printer_loop, NULL);
 }
 
@@ -626,10 +699,11 @@ static void forked_child_init(void)
 			cleanup_buffer(*pbuffer);
 	}
 
-	spawn_printer_thread();
+	if (printer_thread)
+		spawn_printer_thread();
 }
 
-static __attribute__ ((constructor)) void __rt_print_init(void)
+static __attribute__((constructor)) void __rt_print_init(void)
 {
 	const char *value_str;
 	unsigned long long period;
@@ -725,7 +799,6 @@ static __attribute__ ((constructor)) void __rt_print_init(void)
 
 	pthread_cond_init(&printer_wakeup, NULL);
 
-	spawn_printer_thread();
 	pthread_atfork(NULL, NULL, forked_child_init);
 }
 
