@@ -154,15 +154,13 @@ int __wrap_pthread_mutex_lock(pthread_mutex_t *mutex)
 	xnhandle_t cur;
 
 	cur = xeno_get_current();
-	if (cur == XN_NO_HANDLE)
+	if (unlikely(cur == XN_NO_HANDLE))
 		return EPERM;
 
-	status = xeno_get_current_mode();
-
 	if (unlikely(cb_try_read_lock(&shadow->lock, s)))
 		return EINVAL;
 
-	if (shadow->magic != PSE51_MUTEX_MAGIC) {
+	if (unlikely(shadow->magic != PSE51_MUTEX_MAGIC)) {
 		err = -EINVAL;
 		goto out;
 	}
@@ -172,34 +170,37 @@ int __wrap_pthread_mutex_lock(pthread_mutex_t *mutex)
 	 * order to handle the auto-relax feature, so we must always
 	 * obtain them via a syscall.
 	 */
-	if (likely(!(status & (XNRELAX|XNOTHER)))) {
-		err = xnsynch_fast_acquire(get_ownerp(shadow), cur);
+	status = xeno_get_current_mode();
+	if (unlikely(status & (XNRELAX|XNOTHER)))
+		goto do_syscall;
 
-		if (likely(!err)) {
-			shadow->lockcnt = 1;
-			cb_read_unlock(&shadow->lock, s);
-			return 0;
-		}
+	err = xnsynch_fast_acquire(get_ownerp(shadow), cur);
+	if (likely(!err)) {
+		shadow->lockcnt = 1;
+		cb_read_unlock(&shadow->lock, s);
+		return 0;
+	}
 
-		if (err == -EBUSY)
-			switch(shadow->attr.type) {
-			case PTHREAD_MUTEX_NORMAL:
-				break;
+	if (err == -EBUSY)
+		switch(shadow->attr.type) {
+		case PTHREAD_MUTEX_NORMAL:
+			break;
 
-			case PTHREAD_MUTEX_ERRORCHECK:
-				err = -EDEADLK;
-				goto out;
+		case PTHREAD_MUTEX_ERRORCHECK:
+			err = -EDEADLK;
+			goto out;
 
-			case PTHREAD_MUTEX_RECURSIVE:
-				if (shadow->lockcnt == UINT_MAX) {
-					err = -EAGAIN;
-					goto out;
-				}
-				++shadow->lockcnt;
-				err = 0;
+		case PTHREAD_MUTEX_RECURSIVE:
+			if (shadow->lockcnt == UINT_MAX) {
+				err = -EAGAIN;
 				goto out;
 			}
+			++shadow->lockcnt;
+			err = 0;
+			goto out;
 		}
+
+  do_syscall:
 #endif /* CONFIG_XENO_FASTSYNCH */
 
 	do {
@@ -226,48 +227,49 @@ int __wrap_pthread_mutex_timedlock(pthread_mutex_t *mutex,
 	xnhandle_t cur;
 
 	cur = xeno_get_current();
-	if (cur == XN_NO_HANDLE)
+	if (unlikely(cur == XN_NO_HANDLE))
 		return EPERM;
 
-	status = xeno_get_current_mode();
-
 	if (unlikely(cb_try_read_lock(&shadow->lock, s)))
 		return EINVAL;
 
-	if (shadow->magic != PSE51_MUTEX_MAGIC) {
+	if (unlikely(shadow->magic != PSE51_MUTEX_MAGIC)) {
 		err = -EINVAL;
 		goto out;
 	}
 
 	/* See __wrap_pthread_mutex_lock() */
-	if (likely(!(status & (XNRELAX|XNOTHER)))) {
-		err = xnsynch_fast_acquire(get_ownerp(shadow), cur);
+	status = xeno_get_current_mode();
+	if (unlikely(status & (XNRELAX|XNOTHER)))
+		goto do_syscall;
 
-		if (likely(!err)) {
-			shadow->lockcnt = 1;
-			cb_read_unlock(&shadow->lock, s);
-			return 0;
-		}
+	err = xnsynch_fast_acquire(get_ownerp(shadow), cur);
+	if (likely(!err)) {
+		shadow->lockcnt = 1;
+		cb_read_unlock(&shadow->lock, s);
+		return 0;
+	}
 
-		if (err == -EBUSY)
-			switch(shadow->attr.type) {
-			case PTHREAD_MUTEX_NORMAL:
-				break;
+	if (err == -EBUSY)
+		switch(shadow->attr.type) {
+		case PTHREAD_MUTEX_NORMAL:
+			break;
 
-			case PTHREAD_MUTEX_ERRORCHECK:
-				err = -EDEADLK;
-				goto out;
+		case PTHREAD_MUTEX_ERRORCHECK:
+			err = -EDEADLK;
+			goto out;
 
-			case PTHREAD_MUTEX_RECURSIVE:
-				if (shadow->lockcnt == UINT_MAX) {
-					err = -EAGAIN;
-					goto out;
-				}
-
-				++shadow->lockcnt;
+		case PTHREAD_MUTEX_RECURSIVE:
+			if (shadow->lockcnt == UINT_MAX) {
+				err = -EAGAIN;
 				goto out;
 			}
-	}
+
+			++shadow->lockcnt;
+			goto out;
+		}
+
+  do_syscall:
 #endif /* CONFIG_XENO_FASTSYNCH */
 
 	do {
@@ -290,17 +292,15 @@ int __wrap_pthread_mutex_trylock(pthread_mutex_t *mutex)
 	int err;
 
 #ifdef CONFIG_XENO_FASTSYNCH
+	extern int __wrap_clock_gettime(clockid_t clock_id, struct timespec *tp);
+	struct timespec ts;
 	unsigned long status;
 	xnhandle_t cur;
 
 	cur = xeno_get_current();
-	if (cur == XN_NO_HANDLE)
+	if (unlikely(cur == XN_NO_HANDLE))
 		return EPERM;
 
-	status = xeno_get_current_mode();
-	if (unlikely(status & XNOTHER))
-		goto do_syscall;
-
 	if (unlikely(cb_try_read_lock(&shadow->lock, s)))
 		return EINVAL;
 
@@ -309,6 +309,10 @@ int __wrap_pthread_mutex_trylock(pthread_mutex_t *mutex)
 		goto out;
 	}
 
+	status = xeno_get_current_mode();
+	if (unlikely(status & XNOTHER))
+		goto do_syscall;
+
 	if (unlikely(status & XNRELAX)) {
 		do {
 			err = XENOMAI_SYSCALL1(__xn_sys_migrate,
@@ -342,13 +346,25 @@ int __wrap_pthread_mutex_trylock(pthread_mutex_t *mutex)
 	return -err;
 
 do_syscall:
-#endif /* !CONFIG_XENO_FASTSYNCH */
+	__wrap_clock_gettime(CLOCK_REALTIME, &ts);
+	do {
+		err = XENOMAI_SKINCALL2(__pse51_muxid,
+					__pse51_mutex_timedlock, shadow, &ts);
+	} while (err == -EINTR);
+	if (err == -ETIMEDOUT || err == -EDEADLK)
+		err = -EBUSY;
+
+	cb_read_unlock(&shadow->lock, s);
+
+#else /* !CONFIG_XENO_FASTSYNCH */
 
 	do {
 		err = XENOMAI_SKINCALL1(__pse51_muxid,
 					__pse51_mutex_trylock, shadow);
 	} while (err == -EINTR);
 
+#endif /* !CONFIG_XENO_FASTSYNCH */
+
 	return -err;
 }
 
@@ -367,8 +383,6 @@ int __wrap_pthread_mutex_unlock(pthread_mutex_t *mutex)
 	if (cur == XN_NO_HANDLE)
 		return EPERM;
 
-	status = xeno_get_current_mode();
-
 	if (unlikely(cb_try_read_lock(&shadow->lock, s)))
 		return EINVAL;
 
@@ -377,6 +391,7 @@ int __wrap_pthread_mutex_unlock(pthread_mutex_t *mutex)
 		goto out_err;
 	}
 
+	status = xeno_get_current_mode();
 	if (unlikely(status & XNOTHER))
 		goto do_syscall;
 
