@@ -175,6 +175,8 @@ static void *__pthread_trampoline(void *arg)
 	policy = iargs->policy;
 	parent_prio = iargs->parent_prio;
 
+	__real_pthread_setschedparam(pthread_self(), policy, &param);
+
 	/* Do _not_ inline the call to pthread_self() in the syscall
 	   macro: this trashes the syscall regs on some archs. */
 	err = XENOMAI_SKINCALL4(__pse51_muxid, __pse51_thread_create, tid,
@@ -314,6 +316,14 @@ int pthread_set_name_np(pthread_t thread, const char *name)
 				  __pse51_thread_set_name, thread, name);
 }
 
+int sched_setconfig_np(int cpu, int policy,
+		       union sched_config *config, size_t len)
+{
+	return -XENOMAI_SKINCALL4(__pse51_muxid,
+				  __pse51_sched_setconfig_np,
+				  cpu, policy, config, len);
+}
+
 int __wrap_pthread_kill(pthread_t thread, int sig)
 {
 	int err;
