@@ -26,6 +26,7 @@
 #include <memory.h>
 #include <string.h>
 #include <psos+/psos.h>
+#include <psos+/long_names.h>
 #include <asm-generic/bits/sigshadow.h>
 #include <asm-generic/bits/current.h>
 #include <asm-generic/stack.h>
@@ -62,8 +63,10 @@ static void *psos_task_trampoline(void *cookie)
 {
 	struct psos_task_iargs *iargs = (struct psos_task_iargs *)cookie;
 	void (*entry)(u_long, u_long, u_long, u_long);
-	u_long dummy_args[4] = { 0, 0, 0, 0 }, *targs;
+	volatile pthread_t tid = pthread_self();
 	struct psos_arg_bulk bulk;
+	unsigned long mode_offset;
+	u_long handle, targs[4];
 	long err;
 
 	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
@@ -72,8 +75,8 @@ static void *psos_task_trampoline(void *cookie)
 	bulk.a1 = (u_long)iargs->name;
 	bulk.a2 = (u_long)iargs->prio;
 	bulk.a3 = (u_long)iargs->flags;
-	bulk.a4 = (u_long)xeno_init_current_mode();
-	bulk.a5 = (u_long)pthread_self();
+	bulk.a4 = (u_long)&mode_offset;
+	bulk.a5 = (u_long)tid;
 
 	if (!bulk.a4) {
 		err = -ENOMEM;
@@ -87,24 +90,28 @@ static void *psos_task_trampoline(void *cookie)
 		goto fail;
 
 	xeno_set_current();
+	xeno_set_current_mode(mode_offset);
 
 	/* Wait on the barrier for the task to be started. The barrier
 	   could be released in order to process Linux signals while the
 	   Xenomai shadow is still dormant; in such a case, resume wait. */
 
 	do
-		err = XENOMAI_SYSCALL2(__xn_sys_barrier, &entry, &targs);
+		err = XENOMAI_SYSCALL2(__xn_sys_barrier, &entry, &handle);
 	while (err == -EINTR);
+	if (err)
+		goto fail;
 
-	if (!err) {
-		if (targs == NULL)
-			targs = dummy_args;
-		entry(targs[0], targs[1], targs[2], targs[3]);
-	}
+	err = XENOMAI_SKINCALL2(__psos_muxid,
+				__psos_t_getargs, handle, targs);
+	if (err)
+		goto fail;
+
+	entry(targs[0], targs[1], targs[2], targs[3]);
 
       fail:
 
-	pthread_exit((void *)err);
+	return (void *)err;
 }
 
 u_long t_create(const char *name,
@@ -118,10 +125,13 @@ u_long t_create(const char *name,
 	xncompletion_t completion;
 	struct sched_param param;
 	pthread_attr_t thattr;
+	char short_name[5];
 	pthread_t thid;
 	int policy;
 	long err;
 
+	name = __psos_maybe_short_name(short_name, name);
+
 	/* Migrate this thread to the Linux domain since we are about
 	   to issue a series of regular kernel syscalls in order to
 	   create the new Linux thread, which in turn will be mapped
@@ -170,6 +180,7 @@ u_long t_shadow(const char *name, /* Xenomai extension. */
 		u_long *tid_r)
 {
 	struct psos_arg_bulk bulk;
+	unsigned long mode_offset;
 	int ret;
 
 	xeno_fault_stack();
@@ -180,12 +191,14 @@ u_long t_shadow(const char *name, /* Xenomai extension. */
 	bulk.a1 = (u_long)name;
 	bulk.a2 = (u_long)prio;
 	bulk.a3 = (u_long)flags;
-	bulk.a4 = (u_long)xeno_init_current_mode();
+	bulk.a4 = (u_long)&mode_offset;
 	bulk.a5 = (u_long)pthread_self();
 
 	ret = XENOMAI_SKINCALL3(__psos_muxid, __psos_t_create, &bulk, tid_r, NULL);
-	if (!ret)
+	if (!ret) {
 		xeno_set_current();
+		xeno_set_current_mode(mode_offset);
+	}
 
 	return ret;
 }
@@ -260,6 +273,10 @@ u_long t_getreg(u_long tid, u_long regnum, u_long *regvalue_r)
 
 u_long t_ident(const char *name, u_long nodeno, u_long *tid_r)
 {
+	char short_name[5];
+
+	name = __psos_maybe_short_name(short_name, name);
+
 	return XENOMAI_SKINCALL2(__psos_muxid, __psos_t_ident, name, tid_r);
 }
 
