@@ -11,13 +11,18 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <errno.h>
 #include <stdarg.h>
+
+#include <unistd.h>
+#include <signal.h>
 #include <sys/mman.h>
 #include <pthread.h>
-#include <native/timer.h>
+#ifndef __UCLIBC__
+#include <execinfo.h>
+#endif /* !__UCLIBC__ */
 
+#include <native/timer.h>
 #ifndef XENO_POSIX
 #include <native/task.h>
 #include <native/mutex.h>
@@ -41,6 +46,7 @@
 #define THREAD_DETACH		11
 #define THREAD_CREATE		12
 #define THREAD_JOIN		13
+#define THREAD_RENICE           14
 
 #define NS_PER_MS	1000000
 
@@ -54,6 +60,32 @@ typedef RT_TASK thread_t;
 typedef RT_COND cond_t;
 #endif /* __NATIVE_SKIN__ */
 
+static const char *reason_str[] = {
+	[SIGDEBUG_UNDEFINED] = "undefined",
+	[SIGDEBUG_MIGRATE_SIGNAL] = "received signal",
+	[SIGDEBUG_MIGRATE_SYSCALL] = "invoked syscall",
+	[SIGDEBUG_MIGRATE_FAULT] = "triggered fault",
+	[SIGDEBUG_MIGRATE_PRIOINV] = "affected by priority inversion",
+	[SIGDEBUG_NOMLOCK] = "missing mlockall",
+	[SIGDEBUG_WATCHDOG] = "runaway thread",
+};
+
+void sigdebug(int sig, siginfo_t *si, void *context)
+{
+	unsigned int reason = si->si_value.sival_int;
+	void *bt[32];
+	int nentries;
+
+	printf("\nSIGDEBUG received, reason %d: %s\n", reason,
+	       reason <= SIGDEBUG_WATCHDOG ? reason_str[reason] : "<unknown>");
+#ifndef __UCLIBC__
+	/* Dump a backtrace of the frame which caused the switch to
+	   secondary mode: */
+	nentries = backtrace(bt,sizeof(bt) / sizeof(bt[0]));
+	backtrace_symbols_fd(bt,nentries,fileno(stdout));
+#endif /* !__UCLIBC__ */
+}
+
 void timespec_add(struct timespec *ts, unsigned long long value)
 {
 	ts->tv_sec += value / 1000000000;
@@ -110,19 +142,19 @@ void check_current_prio(int expected_prio)
 	}
 }
 
-void check_current_mode(int expected_primary_mode)
+void check_current_mode(int mask, int expected_value)
 {
-	int current_in_primary;
+	int current_mode;
 
 	/* This is a unit test, and in this circonstance, we are allowed to
 	   call xeno_get_current_mode. But please do not do that in your
 	   own code. */
-	current_in_primary = !(xeno_get_current_mode() & XNRELAX);
+	current_mode = xeno_get_current_mode() & mask;
 
-	if (current_in_primary != expected_primary_mode) {
+	if (current_mode != expected_value) {
 		fprintf(stderr,
-			"FAILURE: current mode (%d) != expected mode (%d)\n",
-			current_in_primary, expected_primary_mode);
+			"FAILURE: current mode (%x) != expected mode (%x)\n",
+			current_mode, expected_value);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -270,8 +302,11 @@ int dispatch(const char *service_name,
 #ifdef XENO_POSIX
 		thread = va_arg(ap, pthread_t *);
 		pthread_attr_init(&threadattr);
-		pthread_attr_setschedpolicy(&threadattr, SCHED_FIFO);
 		param.sched_priority = va_arg(ap, int);
+		if (param.sched_priority)
+			pthread_attr_setschedpolicy(&threadattr, SCHED_FIFO);
+		else
+			pthread_attr_setschedpolicy(&threadattr, SCHED_OTHER);
 		pthread_attr_setschedparam(&threadattr, &param);
 		pthread_attr_setinheritsched(&threadattr,
 					     PTHREAD_EXPLICIT_SCHED);
@@ -298,6 +333,23 @@ int dispatch(const char *service_name,
 #endif /* __NATIVE_SKIN__ */
 		break;
 
+	case THREAD_RENICE:
+#ifdef XENO_POSIX
+		param.sched_priority = va_arg(ap, int);
+		if (param.sched_priority)
+			status = pthread_setschedparam(pthread_self(),
+						       SCHED_FIFO, &param);
+		else
+			status = pthread_setschedparam(pthread_self(),
+						       SCHED_OTHER, &param);
+#else /* __NATIVE_SKIN__ */
+		prio = va_arg(ap, int);
+		status = -rt_task_set_priority(rt_task_self(), prio);
+		if (status < 0)
+			status = 0;
+#endif /* __NATIVE_SKIN__ */
+		break;
+
 	default:
 		fprintf(stderr, "Unknown service %i.\n", service_type);
 		exit(EXIT_FAILURE);
@@ -484,15 +536,19 @@ void mode_switch(void)
 {
 	mutex_t mutex;
 
+	/* Cause a switch to secondary mode */
+#ifdef XENO_POSIX
+	__real_sched_yield();
+#endif
 	fprintf(stderr, "mode_switch\n");
 
 	dispatch("switch mutex_init", MUTEX_CREATE, 1, 0, &mutex, 1, 0);
 
-	check_current_mode(0);
+	check_current_mode(XNRELAX, XNRELAX);
 
 	dispatch("switch mutex_lock", MUTEX_LOCK, 1, 0, &mutex);
 
-	check_current_mode(1);
+	check_current_mode(XNRELAX, 0);
 
 	dispatch("switch mutex_unlock", MUTEX_UNLOCK, 1, 0, &mutex);
 	dispatch("switch mutex_destroy", MUTEX_DESTROY, 1, 0, &mutex);
@@ -787,6 +843,50 @@ void recursive_condwait(void)
 	dispatch("rec_condwait join", THREAD_JOIN, 1, 0, &cond_signaler_tid);
 }
 
+void nrt_lock(void *cookie)
+{
+	mutex_t *mutex = (mutex_t *)cookie;
+
+	/* Check that XNOTHER flag gets cleared and set back when
+	   changing priority */
+	check_current_mode(XNRELAX | XNOTHER, XNRELAX | XNOTHER);
+	check_current_prio(0);
+	dispatch("auto_switchback renice 1", THREAD_RENICE, 1, 0, 1);
+	check_current_mode(XNOTHER, 0);
+	check_current_prio(1);
+	dispatch("auto_switchback renice 2", THREAD_RENICE, 1, 0, 0);
+	check_current_mode(XNRELAX | XNOTHER, XNRELAX | XNOTHER);
+	check_current_prio(0);
+
+	/* Check mode changes for auto-switchback threads while using
+	   mutexes with priority inheritance */
+	dispatch("auto_switchback mutex_lock 1", MUTEX_LOCK, 1, 0, mutex);
+	check_current_mode(XNRELAX, 0);
+	ms_sleep(11);
+	check_current_prio(2);
+	dispatch("auto_switchback mutex_unlock 1", MUTEX_UNLOCK, 1, 0, mutex);
+	check_current_mode(XNRELAX | XNOTHER, XNRELAX | XNOTHER);
+}
+
+void auto_switchback(void)
+{
+	thread_t nrt_lock_tid;
+	mutex_t mutex;
+
+	fprintf(stderr, "auto_switchback\n");
+
+	dispatch("auto_switchback mutex_init", MUTEX_CREATE, 1, 0, &mutex, 1,
+		 PTHREAD_MUTEX_RECURSIVE);
+	dispatch("auto_switchback nrt thread_create", THREAD_CREATE, 1, 0,
+		 &nrt_lock_tid, 0, nrt_lock, &mutex);
+	ms_sleep(11);
+	dispatch("auto_switchback mutex_lock 2", MUTEX_LOCK, 1, 0, &mutex);
+	dispatch("auto_switchback mutex_unlock 2", MUTEX_UNLOCK, 1, 0, &mutex);
+
+	dispatch("auto_switchback join", THREAD_JOIN, 1, 0, &nrt_lock_tid);
+	dispatch("auto_switchback mutex_destroy", MUTEX_DESTROY, 1, 0, &mutex);
+}
+
 int main(void)
 {
 #ifdef XENO_POSIX
@@ -794,9 +894,15 @@ int main(void)
 #else /* __NATIVE_SKIN__ */
 	RT_TASK main_tid;
 #endif /* __NATIVE_SKIN__ */
+	struct sigaction sa;
 
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 
+	sigemptyset(&sa.sa_mask);
+	sa.sa_sigaction = sigdebug;
+	sa.sa_flags = SA_SIGINFO;
+	sigaction(SIGDEBUG, &sa, NULL);
+
 	/* Set scheduling parameters for the current process */
 #ifdef XENO_POSIX
 	sparam.sched_priority = 2;
@@ -816,6 +922,7 @@ int main(void)
 	deny_stealing();
 	simple_condwait();
 	recursive_condwait();
+	auto_switchback();
 	fprintf(stderr, "Test OK\n");
 	return 0;
 }
